/* localsearch-albums.c generated by valac 0.56.19, the Vala compiler
 * generated from localsearch-albums.vala, do not modify */

/*
 * Copyright (C) 2009,2010 Nokia Corporation.
 *
 * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
 *                               <zeeshan.ali@nokia.com>
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "rygel-localsearch.h"
#include <glib-object.h>
#include <glib.h>
#include "rygel-server.h"
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	RYGEL_LOCAL_SEARCH_ALBUMS_0_PROPERTY,
	RYGEL_LOCAL_SEARCH_ALBUMS_NUM_PROPERTIES
};
static GParamSpec* rygel_local_search_albums_properties[RYGEL_LOCAL_SEARCH_ALBUMS_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))

static gpointer rygel_local_search_albums_parent_class = NULL;

static GType rygel_local_search_albums_get_type_once (void);

RygelLocalSearchAlbums*
rygel_local_search_albums_construct (GType object_type,
                                     RygelLocalSearchMusic* parent)
{
	RygelLocalSearchAlbums * self = NULL;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	RygelLocalSearchItemFactory* _tmp4_;
	g_return_val_if_fail (parent != NULL, NULL);
	_tmp0_ = rygel_media_object_get_id ((RygelMediaObject*) parent);
	_tmp1_ = _tmp0_;
	_tmp2_ = g_strconcat (_tmp1_, "Albums", NULL);
	_tmp3_ = _tmp2_;
	_tmp4_ = ((RygelLocalSearchCategoryContainer*) parent)->item_factory;
	self = (RygelLocalSearchAlbums*) rygel_local_search_metadata_values_construct (object_type, _tmp3_, (RygelMediaContainer*) parent, _ ("Albums"), _tmp4_, "upnp:album", RYGEL_MEDIA_CONTAINER_MUSIC_ALBUM);
	_g_free0 (_tmp3_);
	return self;
}

RygelLocalSearchAlbums*
rygel_local_search_albums_new (RygelLocalSearchMusic* parent)
{
	return rygel_local_search_albums_construct (RYGEL_LOCAL_SEARCH_TYPE_ALBUMS, parent);
}

static void
rygel_local_search_albums_class_init (RygelLocalSearchAlbumsClass * klass,
                                      gpointer klass_data)
{
	rygel_local_search_albums_parent_class = g_type_class_peek_parent (klass);
}

static void
rygel_local_search_albums_instance_init (RygelLocalSearchAlbums * self,
                                         gpointer klass)
{
}

/**
 * Container listing Music albums as containers.
 */
 G_GNUC_NO_INLINE static GType
rygel_local_search_albums_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelLocalSearchAlbumsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_local_search_albums_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RygelLocalSearchAlbums), 0, (GInstanceInitFunc) rygel_local_search_albums_instance_init, NULL };
	GType rygel_local_search_albums_type_id;
	rygel_local_search_albums_type_id = g_type_register_static (RYGEL_LOCAL_SEARCH_TYPE_METADATA_VALUES, "RygelLocalSearchAlbums", &g_define_type_info, 0);
	return rygel_local_search_albums_type_id;
}

GType
rygel_local_search_albums_get_type (void)
{
	static volatile gsize rygel_local_search_albums_type_id__once = 0;
	if (g_once_init_enter (&rygel_local_search_albums_type_id__once)) {
		GType rygel_local_search_albums_type_id;
		rygel_local_search_albums_type_id = rygel_local_search_albums_get_type_once ();
		g_once_init_leave (&rygel_local_search_albums_type_id__once, rygel_local_search_albums_type_id);
	}
	return rygel_local_search_albums_type_id__once;
}

