/* rygel-ruih-service-error.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-ruih-service-error.vala, do not modify */

/*
 * Copyright (C) 2013  Cable Television Laboratories, Inc.
 *
 * Author: Neha Shanbhag <N.Shanbhag@cablelabs.com>
 * Contact: http://www.cablelabs.com/
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
/* These error codes are returned to applications and
    must be defined in the spec.
 */

#include "rygel-ruih.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

GQuark
rygel_ruih_service_error_quark (void)
{
	return g_quark_from_static_string ("rygel-ruih-service-error-quark");
}

 G_GNUC_NO_INLINE static GType
rygel_ruih_service_error_get_type_once (void)
{
	static const GEnumValue values[] = {{RYGEL_RUIH_SERVICE_ERROR_OPERATION_REJECTED, "RYGEL_RUIH_SERVICE_ERROR_OPERATION_REJECTED", "operation-rejected"}, {RYGEL_RUIH_SERVICE_ERROR_INVALID_FILTER, "RYGEL_RUIH_SERVICE_ERROR_INVALID_FILTER", "invalid-filter"}, {0, NULL, NULL}};
	GType rygel_ruih_service_error_type_id;
	rygel_ruih_service_error_type_id = g_enum_register_static ("RygelRuihServiceError", values);
	return rygel_ruih_service_error_type_id;
}

GType
rygel_ruih_service_error_get_type (void)
{
	static volatile gsize rygel_ruih_service_error_type_id__once = 0;
	if (g_once_init_enter (&rygel_ruih_service_error_type_id__once)) {
		GType rygel_ruih_service_error_type_id;
		rygel_ruih_service_error_type_id = rygel_ruih_service_error_get_type_once ();
		g_once_init_leave (&rygel_ruih_service_error_type_id__once, rygel_ruih_service_error_type_id);
	}
	return rygel_ruih_service_error_type_id__once;
}

