/* rygel-localsearch.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __SRC_PLUGINS_LOCALSEARCH_RYGEL_LOCALSEARCH_H__
#define __SRC_PLUGINS_LOCALSEARCH_RYGEL_LOCALSEARCH_H__

#include "rygel-server.h"
#include <glib-object.h>
#include <tinysparql.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <gee.h>
#include <gio/gio.h>
#include "rygel-core.h"

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif
#define RYGEL_LOCAL_SEARCH_SELECTION_QUERY_ITEM_VARIABLE "?item"
#define RYGEL_LOCAL_SEARCH_SELECTION_QUERY_STORAGE_VARIABLE "?storage"
#define RYGEL_LOCAL_SEARCH_PLUGIN_NAME "LocalSearch"

#define RYGEL_LOCAL_SEARCH_TYPE_ROOT_CONTAINER (rygel_local_search_root_container_get_type ())
#define RYGEL_LOCAL_SEARCH_ROOT_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_ROOT_CONTAINER, RygelLocalSearchRootContainer))
#define RYGEL_LOCAL_SEARCH_ROOT_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_ROOT_CONTAINER, RygelLocalSearchRootContainerClass))
#define RYGEL_LOCAL_SEARCH_IS_ROOT_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_ROOT_CONTAINER))
#define RYGEL_LOCAL_SEARCH_IS_ROOT_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_ROOT_CONTAINER))
#define RYGEL_LOCAL_SEARCH_ROOT_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_ROOT_CONTAINER, RygelLocalSearchRootContainerClass))

typedef struct _RygelLocalSearchRootContainer RygelLocalSearchRootContainer;
typedef struct _RygelLocalSearchRootContainerClass RygelLocalSearchRootContainerClass;
typedef struct _RygelLocalSearchRootContainerPrivate RygelLocalSearchRootContainerPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_CONTAINER (rygel_local_search_category_container_get_type ())
#define RYGEL_LOCAL_SEARCH_CATEGORY_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_CONTAINER, RygelLocalSearchCategoryContainer))
#define RYGEL_LOCAL_SEARCH_CATEGORY_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_CONTAINER, RygelLocalSearchCategoryContainerClass))
#define RYGEL_LOCAL_SEARCH_IS_CATEGORY_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_CONTAINER))
#define RYGEL_LOCAL_SEARCH_IS_CATEGORY_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_CONTAINER))
#define RYGEL_LOCAL_SEARCH_CATEGORY_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_CONTAINER, RygelLocalSearchCategoryContainerClass))

typedef struct _RygelLocalSearchCategoryContainer RygelLocalSearchCategoryContainer;
typedef struct _RygelLocalSearchCategoryContainerClass RygelLocalSearchCategoryContainerClass;
typedef struct _RygelLocalSearchCategoryContainerPrivate RygelLocalSearchCategoryContainerPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_ITEM_FACTORY (rygel_local_search_item_factory_get_type ())
#define RYGEL_LOCAL_SEARCH_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_ITEM_FACTORY, RygelLocalSearchItemFactory))
#define RYGEL_LOCAL_SEARCH_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_ITEM_FACTORY, RygelLocalSearchItemFactoryClass))
#define RYGEL_LOCAL_SEARCH_IS_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_IS_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_ITEM_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_ITEM_FACTORY, RygelLocalSearchItemFactoryClass))

typedef struct _RygelLocalSearchItemFactory RygelLocalSearchItemFactory;
typedef struct _RygelLocalSearchItemFactoryClass RygelLocalSearchItemFactoryClass;

#define RYGEL_LOCAL_SEARCH_TYPE_MUSIC (rygel_local_search_music_get_type ())
#define RYGEL_LOCAL_SEARCH_MUSIC(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_MUSIC, RygelLocalSearchMusic))
#define RYGEL_LOCAL_SEARCH_MUSIC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_MUSIC, RygelLocalSearchMusicClass))
#define RYGEL_LOCAL_SEARCH_IS_MUSIC(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_MUSIC))
#define RYGEL_LOCAL_SEARCH_IS_MUSIC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_MUSIC))
#define RYGEL_LOCAL_SEARCH_MUSIC_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_MUSIC, RygelLocalSearchMusicClass))

typedef struct _RygelLocalSearchMusic RygelLocalSearchMusic;
typedef struct _RygelLocalSearchMusicClass RygelLocalSearchMusicClass;
typedef struct _RygelLocalSearchMusicPrivate RygelLocalSearchMusicPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_VIDEOS (rygel_local_search_videos_get_type ())
#define RYGEL_LOCAL_SEARCH_VIDEOS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_VIDEOS, RygelLocalSearchVideos))
#define RYGEL_LOCAL_SEARCH_VIDEOS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_VIDEOS, RygelLocalSearchVideosClass))
#define RYGEL_LOCAL_SEARCH_IS_VIDEOS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_VIDEOS))
#define RYGEL_LOCAL_SEARCH_IS_VIDEOS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_VIDEOS))
#define RYGEL_LOCAL_SEARCH_VIDEOS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_VIDEOS, RygelLocalSearchVideosClass))

typedef struct _RygelLocalSearchVideos RygelLocalSearchVideos;
typedef struct _RygelLocalSearchVideosClass RygelLocalSearchVideosClass;
typedef struct _RygelLocalSearchVideosPrivate RygelLocalSearchVideosPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_PICTURES (rygel_local_search_pictures_get_type ())
#define RYGEL_LOCAL_SEARCH_PICTURES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_PICTURES, RygelLocalSearchPictures))
#define RYGEL_LOCAL_SEARCH_PICTURES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_PICTURES, RygelLocalSearchPicturesClass))
#define RYGEL_LOCAL_SEARCH_IS_PICTURES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_PICTURES))
#define RYGEL_LOCAL_SEARCH_IS_PICTURES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_PICTURES))
#define RYGEL_LOCAL_SEARCH_PICTURES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_PICTURES, RygelLocalSearchPicturesClass))

typedef struct _RygelLocalSearchPictures RygelLocalSearchPictures;
typedef struct _RygelLocalSearchPicturesClass RygelLocalSearchPicturesClass;
typedef struct _RygelLocalSearchPicturesPrivate RygelLocalSearchPicturesPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_METADATA_CONTAINER (rygel_local_search_metadata_container_get_type ())
#define RYGEL_LOCAL_SEARCH_METADATA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_CONTAINER, RygelLocalSearchMetadataContainer))
#define RYGEL_LOCAL_SEARCH_METADATA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_METADATA_CONTAINER, RygelLocalSearchMetadataContainerClass))
#define RYGEL_LOCAL_SEARCH_IS_METADATA_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_CONTAINER))
#define RYGEL_LOCAL_SEARCH_IS_METADATA_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_METADATA_CONTAINER))
#define RYGEL_LOCAL_SEARCH_METADATA_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_CONTAINER, RygelLocalSearchMetadataContainerClass))

typedef struct _RygelLocalSearchMetadataContainer RygelLocalSearchMetadataContainer;
typedef struct _RygelLocalSearchMetadataContainerClass RygelLocalSearchMetadataContainerClass;
typedef struct _RygelLocalSearchMetadataContainerPrivate RygelLocalSearchMetadataContainerPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_QUERY (rygel_local_search_query_get_type ())
#define RYGEL_LOCAL_SEARCH_QUERY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY, RygelLocalSearchQuery))
#define RYGEL_LOCAL_SEARCH_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_QUERY, RygelLocalSearchQueryClass))
#define RYGEL_LOCAL_SEARCH_IS_QUERY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY))
#define RYGEL_LOCAL_SEARCH_IS_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_QUERY))
#define RYGEL_LOCAL_SEARCH_QUERY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY, RygelLocalSearchQueryClass))

typedef struct _RygelLocalSearchQuery RygelLocalSearchQuery;
typedef struct _RygelLocalSearchQueryClass RygelLocalSearchQueryClass;

#define RYGEL_LOCAL_SEARCH_TYPE_SELECTION_QUERY (rygel_local_search_selection_query_get_type ())
#define RYGEL_LOCAL_SEARCH_SELECTION_QUERY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_SELECTION_QUERY, RygelLocalSearchSelectionQuery))
#define RYGEL_LOCAL_SEARCH_SELECTION_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_SELECTION_QUERY, RygelLocalSearchSelectionQueryClass))
#define RYGEL_LOCAL_SEARCH_IS_SELECTION_QUERY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_SELECTION_QUERY))
#define RYGEL_LOCAL_SEARCH_IS_SELECTION_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_SELECTION_QUERY))
#define RYGEL_LOCAL_SEARCH_SELECTION_QUERY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_SELECTION_QUERY, RygelLocalSearchSelectionQueryClass))

typedef struct _RygelLocalSearchSelectionQuery RygelLocalSearchSelectionQuery;
typedef struct _RygelLocalSearchSelectionQueryClass RygelLocalSearchSelectionQueryClass;

#define RYGEL_LOCAL_SEARCH_TYPE_SEARCH_CONTAINER (rygel_local_search_search_container_get_type ())
#define RYGEL_LOCAL_SEARCH_SEARCH_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_SEARCH_CONTAINER, RygelLocalSearchSearchContainer))
#define RYGEL_LOCAL_SEARCH_SEARCH_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_SEARCH_CONTAINER, RygelLocalSearchSearchContainerClass))
#define RYGEL_LOCAL_SEARCH_IS_SEARCH_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_SEARCH_CONTAINER))
#define RYGEL_LOCAL_SEARCH_IS_SEARCH_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_SEARCH_CONTAINER))
#define RYGEL_LOCAL_SEARCH_SEARCH_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_SEARCH_CONTAINER, RygelLocalSearchSearchContainerClass))

typedef struct _RygelLocalSearchSearchContainer RygelLocalSearchSearchContainer;
typedef struct _RygelLocalSearchSearchContainerClass RygelLocalSearchSearchContainerClass;

#define RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLETS (rygel_local_search_query_triplets_get_type ())
#define RYGEL_LOCAL_SEARCH_QUERY_TRIPLETS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLETS, RygelLocalSearchQueryTriplets))
#define RYGEL_LOCAL_SEARCH_QUERY_TRIPLETS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLETS, RygelLocalSearchQueryTripletsClass))
#define RYGEL_LOCAL_SEARCH_IS_QUERY_TRIPLETS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLETS))
#define RYGEL_LOCAL_SEARCH_IS_QUERY_TRIPLETS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLETS))
#define RYGEL_LOCAL_SEARCH_QUERY_TRIPLETS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLETS, RygelLocalSearchQueryTripletsClass))

typedef struct _RygelLocalSearchQueryTriplets RygelLocalSearchQueryTriplets;
typedef struct _RygelLocalSearchQueryTripletsClass RygelLocalSearchQueryTripletsClass;

#define RYGEL_LOCAL_SEARCH_TYPE_METADATA_VALUES (rygel_local_search_metadata_values_get_type ())
#define RYGEL_LOCAL_SEARCH_METADATA_VALUES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_VALUES, RygelLocalSearchMetadataValues))
#define RYGEL_LOCAL_SEARCH_METADATA_VALUES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_METADATA_VALUES, RygelLocalSearchMetadataValuesClass))
#define RYGEL_LOCAL_SEARCH_IS_METADATA_VALUES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_VALUES))
#define RYGEL_LOCAL_SEARCH_IS_METADATA_VALUES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_METADATA_VALUES))
#define RYGEL_LOCAL_SEARCH_METADATA_VALUES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_VALUES, RygelLocalSearchMetadataValuesClass))

typedef struct _RygelLocalSearchMetadataValues RygelLocalSearchMetadataValues;
typedef struct _RygelLocalSearchMetadataValuesClass RygelLocalSearchMetadataValuesClass;
typedef struct _RygelLocalSearchMetadataValuesPrivate RygelLocalSearchMetadataValuesPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_METADATA_MULTI_VALUES (rygel_local_search_metadata_multi_values_get_type ())
#define RYGEL_LOCAL_SEARCH_METADATA_MULTI_VALUES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_MULTI_VALUES, RygelLocalSearchMetadataMultiValues))
#define RYGEL_LOCAL_SEARCH_METADATA_MULTI_VALUES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_METADATA_MULTI_VALUES, RygelLocalSearchMetadataMultiValuesClass))
#define RYGEL_LOCAL_SEARCH_IS_METADATA_MULTI_VALUES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_MULTI_VALUES))
#define RYGEL_LOCAL_SEARCH_IS_METADATA_MULTI_VALUES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_METADATA_MULTI_VALUES))
#define RYGEL_LOCAL_SEARCH_METADATA_MULTI_VALUES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_METADATA_MULTI_VALUES, RygelLocalSearchMetadataMultiValuesClass))

typedef struct _RygelLocalSearchMetadataMultiValues RygelLocalSearchMetadataMultiValues;
typedef struct _RygelLocalSearchMetadataMultiValuesClass RygelLocalSearchMetadataMultiValuesClass;
typedef struct _RygelLocalSearchMetadataMultiValuesPrivate RygelLocalSearchMetadataMultiValuesPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_TAGS (rygel_local_search_tags_get_type ())
#define RYGEL_LOCAL_SEARCH_TAGS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_TAGS, RygelLocalSearchTags))
#define RYGEL_LOCAL_SEARCH_TAGS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_TAGS, RygelLocalSearchTagsClass))
#define RYGEL_LOCAL_SEARCH_IS_TAGS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_TAGS))
#define RYGEL_LOCAL_SEARCH_IS_TAGS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_TAGS))
#define RYGEL_LOCAL_SEARCH_TAGS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_TAGS, RygelLocalSearchTagsClass))

typedef struct _RygelLocalSearchTags RygelLocalSearchTags;
typedef struct _RygelLocalSearchTagsClass RygelLocalSearchTagsClass;
typedef struct _RygelLocalSearchTagsPrivate RygelLocalSearchTagsPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_YEARS (rygel_local_search_years_get_type ())
#define RYGEL_LOCAL_SEARCH_YEARS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_YEARS, RygelLocalSearchYears))
#define RYGEL_LOCAL_SEARCH_YEARS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_YEARS, RygelLocalSearchYearsClass))
#define RYGEL_LOCAL_SEARCH_IS_YEARS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_YEARS))
#define RYGEL_LOCAL_SEARCH_IS_YEARS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_YEARS))
#define RYGEL_LOCAL_SEARCH_YEARS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_YEARS, RygelLocalSearchYearsClass))

typedef struct _RygelLocalSearchYears RygelLocalSearchYears;
typedef struct _RygelLocalSearchYearsClass RygelLocalSearchYearsClass;
typedef struct _RygelLocalSearchYearsPrivate RygelLocalSearchYearsPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_TITLES (rygel_local_search_titles_get_type ())
#define RYGEL_LOCAL_SEARCH_TITLES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_TITLES, RygelLocalSearchTitles))
#define RYGEL_LOCAL_SEARCH_TITLES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_TITLES, RygelLocalSearchTitlesClass))
#define RYGEL_LOCAL_SEARCH_IS_TITLES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_TITLES))
#define RYGEL_LOCAL_SEARCH_IS_TITLES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_TITLES))
#define RYGEL_LOCAL_SEARCH_TITLES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_TITLES, RygelLocalSearchTitlesClass))

typedef struct _RygelLocalSearchTitles RygelLocalSearchTitles;
typedef struct _RygelLocalSearchTitlesClass RygelLocalSearchTitlesClass;
typedef struct _RygelLocalSearchTitlesPrivate RygelLocalSearchTitlesPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_ALBUMS (rygel_local_search_albums_get_type ())
#define RYGEL_LOCAL_SEARCH_ALBUMS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_ALBUMS, RygelLocalSearchAlbums))
#define RYGEL_LOCAL_SEARCH_ALBUMS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_ALBUMS, RygelLocalSearchAlbumsClass))
#define RYGEL_LOCAL_SEARCH_IS_ALBUMS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_ALBUMS))
#define RYGEL_LOCAL_SEARCH_IS_ALBUMS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_ALBUMS))
#define RYGEL_LOCAL_SEARCH_ALBUMS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_ALBUMS, RygelLocalSearchAlbumsClass))

typedef struct _RygelLocalSearchAlbums RygelLocalSearchAlbums;
typedef struct _RygelLocalSearchAlbumsClass RygelLocalSearchAlbumsClass;
typedef struct _RygelLocalSearchAlbumsPrivate RygelLocalSearchAlbumsPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_ARTISTS (rygel_local_search_artists_get_type ())
#define RYGEL_LOCAL_SEARCH_ARTISTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_ARTISTS, RygelLocalSearchArtists))
#define RYGEL_LOCAL_SEARCH_ARTISTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_ARTISTS, RygelLocalSearchArtistsClass))
#define RYGEL_LOCAL_SEARCH_IS_ARTISTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_ARTISTS))
#define RYGEL_LOCAL_SEARCH_IS_ARTISTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_ARTISTS))
#define RYGEL_LOCAL_SEARCH_ARTISTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_ARTISTS, RygelLocalSearchArtistsClass))

typedef struct _RygelLocalSearchArtists RygelLocalSearchArtists;
typedef struct _RygelLocalSearchArtistsClass RygelLocalSearchArtistsClass;
typedef struct _RygelLocalSearchArtistsPrivate RygelLocalSearchArtistsPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_GENRE (rygel_local_search_genre_get_type ())
#define RYGEL_LOCAL_SEARCH_GENRE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_GENRE, RygelLocalSearchGenre))
#define RYGEL_LOCAL_SEARCH_GENRE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_GENRE, RygelLocalSearchGenreClass))
#define RYGEL_LOCAL_SEARCH_IS_GENRE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_GENRE))
#define RYGEL_LOCAL_SEARCH_IS_GENRE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_GENRE))
#define RYGEL_LOCAL_SEARCH_GENRE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_GENRE, RygelLocalSearchGenreClass))

typedef struct _RygelLocalSearchGenre RygelLocalSearchGenre;
typedef struct _RygelLocalSearchGenreClass RygelLocalSearchGenreClass;
typedef struct _RygelLocalSearchGenrePrivate RygelLocalSearchGenrePrivate;
typedef struct _RygelLocalSearchSearchContainerPrivate RygelLocalSearchSearchContainerPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_NEW (rygel_local_search_new_get_type ())
#define RYGEL_LOCAL_SEARCH_NEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_NEW, RygelLocalSearchNew))
#define RYGEL_LOCAL_SEARCH_NEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_NEW, RygelLocalSearchNewClass))
#define RYGEL_LOCAL_SEARCH_IS_NEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_NEW))
#define RYGEL_LOCAL_SEARCH_IS_NEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_NEW))
#define RYGEL_LOCAL_SEARCH_NEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_NEW, RygelLocalSearchNewClass))

typedef struct _RygelLocalSearchNew RygelLocalSearchNew;
typedef struct _RygelLocalSearchNewClass RygelLocalSearchNewClass;
typedef struct _RygelLocalSearchNewPrivate RygelLocalSearchNewPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_ALL_CONTAINER (rygel_local_search_category_all_container_get_type ())
#define RYGEL_LOCAL_SEARCH_CATEGORY_ALL_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_ALL_CONTAINER, RygelLocalSearchCategoryAllContainer))
#define RYGEL_LOCAL_SEARCH_CATEGORY_ALL_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_ALL_CONTAINER, RygelLocalSearchCategoryAllContainerClass))
#define RYGEL_LOCAL_SEARCH_IS_CATEGORY_ALL_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_ALL_CONTAINER))
#define RYGEL_LOCAL_SEARCH_IS_CATEGORY_ALL_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_ALL_CONTAINER))
#define RYGEL_LOCAL_SEARCH_CATEGORY_ALL_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_CATEGORY_ALL_CONTAINER, RygelLocalSearchCategoryAllContainerClass))

typedef struct _RygelLocalSearchCategoryAllContainer RygelLocalSearchCategoryAllContainer;
typedef struct _RygelLocalSearchCategoryAllContainerClass RygelLocalSearchCategoryAllContainerClass;
typedef struct _RygelLocalSearchCategoryAllContainerPrivate RygelLocalSearchCategoryAllContainerPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_UPN_PPROPERTY_MAP (rygel_local_search_upn_pproperty_map_get_type ())
#define RYGEL_LOCAL_SEARCH_UPN_PPROPERTY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_UPN_PPROPERTY_MAP, RygelLocalSearchUPnPPropertyMap))
#define RYGEL_LOCAL_SEARCH_UPN_PPROPERTY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_UPN_PPROPERTY_MAP, RygelLocalSearchUPnPPropertyMapClass))
#define RYGEL_LOCAL_SEARCH_IS_UPN_PPROPERTY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_UPN_PPROPERTY_MAP))
#define RYGEL_LOCAL_SEARCH_IS_UPN_PPROPERTY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_UPN_PPROPERTY_MAP))
#define RYGEL_LOCAL_SEARCH_UPN_PPROPERTY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_UPN_PPROPERTY_MAP, RygelLocalSearchUPnPPropertyMapClass))

typedef struct _RygelLocalSearchUPnPPropertyMap RygelLocalSearchUPnPPropertyMap;
typedef struct _RygelLocalSearchUPnPPropertyMapClass RygelLocalSearchUPnPPropertyMapClass;
typedef struct _RygelLocalSearchUPnPPropertyMapPrivate RygelLocalSearchUPnPPropertyMapPrivate;
typedef struct _RygelLocalSearchQueryPrivate RygelLocalSearchQueryPrivate;
typedef struct _RygelLocalSearchSelectionQueryPrivate RygelLocalSearchSelectionQueryPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_DELETION_QUERY (rygel_local_search_deletion_query_get_type ())
#define RYGEL_LOCAL_SEARCH_DELETION_QUERY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_DELETION_QUERY, RygelLocalSearchDeletionQuery))
#define RYGEL_LOCAL_SEARCH_DELETION_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_DELETION_QUERY, RygelLocalSearchDeletionQueryClass))
#define RYGEL_LOCAL_SEARCH_IS_DELETION_QUERY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_DELETION_QUERY))
#define RYGEL_LOCAL_SEARCH_IS_DELETION_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_DELETION_QUERY))
#define RYGEL_LOCAL_SEARCH_DELETION_QUERY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_DELETION_QUERY, RygelLocalSearchDeletionQueryClass))

typedef struct _RygelLocalSearchDeletionQuery RygelLocalSearchDeletionQuery;
typedef struct _RygelLocalSearchDeletionQueryClass RygelLocalSearchDeletionQueryClass;
typedef struct _RygelLocalSearchDeletionQueryPrivate RygelLocalSearchDeletionQueryPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_INSERTION_QUERY (rygel_local_search_insertion_query_get_type ())
#define RYGEL_LOCAL_SEARCH_INSERTION_QUERY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_INSERTION_QUERY, RygelLocalSearchInsertionQuery))
#define RYGEL_LOCAL_SEARCH_INSERTION_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_INSERTION_QUERY, RygelLocalSearchInsertionQueryClass))
#define RYGEL_LOCAL_SEARCH_IS_INSERTION_QUERY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_INSERTION_QUERY))
#define RYGEL_LOCAL_SEARCH_IS_INSERTION_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_INSERTION_QUERY))
#define RYGEL_LOCAL_SEARCH_INSERTION_QUERY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_INSERTION_QUERY, RygelLocalSearchInsertionQueryClass))

typedef struct _RygelLocalSearchInsertionQuery RygelLocalSearchInsertionQuery;
typedef struct _RygelLocalSearchInsertionQueryClass RygelLocalSearchInsertionQueryClass;
typedef struct _RygelLocalSearchInsertionQueryPrivate RygelLocalSearchInsertionQueryPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_CLEANUP_QUERY (rygel_local_search_cleanup_query_get_type ())
#define RYGEL_LOCAL_SEARCH_CLEANUP_QUERY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_CLEANUP_QUERY, RygelLocalSearchCleanupQuery))
#define RYGEL_LOCAL_SEARCH_CLEANUP_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_CLEANUP_QUERY, RygelLocalSearchCleanupQueryClass))
#define RYGEL_LOCAL_SEARCH_IS_CLEANUP_QUERY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_CLEANUP_QUERY))
#define RYGEL_LOCAL_SEARCH_IS_CLEANUP_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_CLEANUP_QUERY))
#define RYGEL_LOCAL_SEARCH_CLEANUP_QUERY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_CLEANUP_QUERY, RygelLocalSearchCleanupQueryClass))

typedef struct _RygelLocalSearchCleanupQuery RygelLocalSearchCleanupQuery;
typedef struct _RygelLocalSearchCleanupQueryClass RygelLocalSearchCleanupQueryClass;
typedef struct _RygelLocalSearchCleanupQueryPrivate RygelLocalSearchCleanupQueryPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLET (rygel_local_search_query_triplet_get_type ())
#define RYGEL_LOCAL_SEARCH_QUERY_TRIPLET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLET, RygelLocalSearchQueryTriplet))
#define RYGEL_LOCAL_SEARCH_QUERY_TRIPLET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLET, RygelLocalSearchQueryTripletClass))
#define RYGEL_LOCAL_SEARCH_IS_QUERY_TRIPLET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLET))
#define RYGEL_LOCAL_SEARCH_IS_QUERY_TRIPLET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLET))
#define RYGEL_LOCAL_SEARCH_QUERY_TRIPLET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_QUERY_TRIPLET, RygelLocalSearchQueryTripletClass))

typedef struct _RygelLocalSearchQueryTriplet RygelLocalSearchQueryTriplet;
typedef struct _RygelLocalSearchQueryTripletClass RygelLocalSearchQueryTripletClass;
typedef struct _RygelLocalSearchQueryTripletPrivate RygelLocalSearchQueryTripletPrivate;
typedef struct _RygelLocalSearchQueryTripletsPrivate RygelLocalSearchQueryTripletsPrivate;
typedef struct _RygelLocalSearchItemFactoryPrivate RygelLocalSearchItemFactoryPrivate;
typedef enum  {
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_TRACKER_ID,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_URL,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_PLACE_HOLDER,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_FILE_NAME,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_TITLE,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_DLNA_PROFILE,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_MIME,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_SIZE,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_DATE,
	RYGEL_LOCAL_SEARCH_ITEM_FACTORY_METADATA_LAST_KEY
} RygelLocalSearchItemFactoryMetadata;

#define RYGEL_LOCAL_SEARCH_ITEM_FACTORY_TYPE_METADATA (rygel_local_search_item_factory_metadata_get_type ())

#define RYGEL_LOCAL_SEARCH_TYPE_VIDEO_ITEM_FACTORY (rygel_local_search_video_item_factory_get_type ())
#define RYGEL_LOCAL_SEARCH_VIDEO_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_VIDEO_ITEM_FACTORY, RygelLocalSearchVideoItemFactory))
#define RYGEL_LOCAL_SEARCH_VIDEO_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_VIDEO_ITEM_FACTORY, RygelLocalSearchVideoItemFactoryClass))
#define RYGEL_LOCAL_SEARCH_IS_VIDEO_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_VIDEO_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_IS_VIDEO_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_VIDEO_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_VIDEO_ITEM_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_VIDEO_ITEM_FACTORY, RygelLocalSearchVideoItemFactoryClass))

typedef struct _RygelLocalSearchVideoItemFactory RygelLocalSearchVideoItemFactory;
typedef struct _RygelLocalSearchVideoItemFactoryClass RygelLocalSearchVideoItemFactoryClass;
typedef struct _RygelLocalSearchVideoItemFactoryPrivate RygelLocalSearchVideoItemFactoryPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_MUSIC_ITEM_FACTORY (rygel_local_search_music_item_factory_get_type ())
#define RYGEL_LOCAL_SEARCH_MUSIC_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_MUSIC_ITEM_FACTORY, RygelLocalSearchMusicItemFactory))
#define RYGEL_LOCAL_SEARCH_MUSIC_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_MUSIC_ITEM_FACTORY, RygelLocalSearchMusicItemFactoryClass))
#define RYGEL_LOCAL_SEARCH_IS_MUSIC_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_MUSIC_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_IS_MUSIC_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_MUSIC_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_MUSIC_ITEM_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_MUSIC_ITEM_FACTORY, RygelLocalSearchMusicItemFactoryClass))

typedef struct _RygelLocalSearchMusicItemFactory RygelLocalSearchMusicItemFactory;
typedef struct _RygelLocalSearchMusicItemFactoryClass RygelLocalSearchMusicItemFactoryClass;
typedef struct _RygelLocalSearchMusicItemFactoryPrivate RygelLocalSearchMusicItemFactoryPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_PICTURE_ITEM_FACTORY (rygel_local_search_picture_item_factory_get_type ())
#define RYGEL_LOCAL_SEARCH_PICTURE_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_PICTURE_ITEM_FACTORY, RygelLocalSearchPictureItemFactory))
#define RYGEL_LOCAL_SEARCH_PICTURE_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_PICTURE_ITEM_FACTORY, RygelLocalSearchPictureItemFactoryClass))
#define RYGEL_LOCAL_SEARCH_IS_PICTURE_ITEM_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_PICTURE_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_IS_PICTURE_ITEM_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_PICTURE_ITEM_FACTORY))
#define RYGEL_LOCAL_SEARCH_PICTURE_ITEM_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_PICTURE_ITEM_FACTORY, RygelLocalSearchPictureItemFactoryClass))

typedef struct _RygelLocalSearchPictureItemFactory RygelLocalSearchPictureItemFactory;
typedef struct _RygelLocalSearchPictureItemFactoryClass RygelLocalSearchPictureItemFactoryClass;
typedef struct _RygelLocalSearchPictureItemFactoryPrivate RygelLocalSearchPictureItemFactoryPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_PLUGIN (rygel_local_search_plugin_get_type ())
#define RYGEL_LOCAL_SEARCH_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN, RygelLocalSearchPlugin))
#define RYGEL_LOCAL_SEARCH_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN, RygelLocalSearchPluginClass))
#define RYGEL_LOCAL_SEARCH_IS_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN))
#define RYGEL_LOCAL_SEARCH_IS_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN))
#define RYGEL_LOCAL_SEARCH_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN, RygelLocalSearchPluginClass))

typedef struct _RygelLocalSearchPlugin RygelLocalSearchPlugin;
typedef struct _RygelLocalSearchPluginClass RygelLocalSearchPluginClass;
typedef struct _RygelLocalSearchPluginPrivate RygelLocalSearchPluginPrivate;

#define RYGEL_LOCAL_SEARCH_TYPE_PLUGIN_FACTORY (rygel_local_search_plugin_factory_get_type ())
#define RYGEL_LOCAL_SEARCH_PLUGIN_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN_FACTORY, RygelLocalSearchPluginFactory))
#define RYGEL_LOCAL_SEARCH_PLUGIN_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN_FACTORY, RygelLocalSearchPluginFactoryClass))
#define RYGEL_LOCAL_SEARCH_IS_PLUGIN_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN_FACTORY))
#define RYGEL_LOCAL_SEARCH_IS_PLUGIN_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN_FACTORY))
#define RYGEL_LOCAL_SEARCH_PLUGIN_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_LOCAL_SEARCH_TYPE_PLUGIN_FACTORY, RygelLocalSearchPluginFactoryClass))

typedef struct _RygelLocalSearchPluginFactory RygelLocalSearchPluginFactory;
typedef struct _RygelLocalSearchPluginFactoryClass RygelLocalSearchPluginFactoryClass;
typedef struct _RygelLocalSearchPluginFactoryPrivate RygelLocalSearchPluginFactoryPrivate;

#define TYPE_EVENT (event_get_type ())
typedef struct _Event Event;

#define RYGEL_LOCAL_SEARCH_TYPE_STATS_IFACE (rygel_local_search_stats_iface_get_type ())
#define RYGEL_LOCAL_SEARCH_STATS_IFACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_STATS_IFACE, RygelLocalSearchStatsIface))
#define RYGEL_LOCAL_SEARCH_IS_STATS_IFACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_STATS_IFACE))
#define RYGEL_LOCAL_SEARCH_STATS_IFACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_LOCAL_SEARCH_TYPE_STATS_IFACE, RygelLocalSearchStatsIfaceIface))

typedef struct _RygelLocalSearchStatsIface RygelLocalSearchStatsIface;
typedef struct _RygelLocalSearchStatsIfaceIface RygelLocalSearchStatsIfaceIface;

#define RYGEL_LOCAL_SEARCH_TYPE_STATS_IFACE_PROXY (rygel_local_search_stats_iface_proxy_get_type ())

#define RYGEL_LOCAL_SEARCH_TYPE_RESOURCES_IFACE (rygel_local_search_resources_iface_get_type ())
#define RYGEL_LOCAL_SEARCH_RESOURCES_IFACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_RESOURCES_IFACE, RygelLocalSearchResourcesIface))
#define RYGEL_LOCAL_SEARCH_IS_RESOURCES_IFACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_RESOURCES_IFACE))
#define RYGEL_LOCAL_SEARCH_RESOURCES_IFACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_LOCAL_SEARCH_TYPE_RESOURCES_IFACE, RygelLocalSearchResourcesIfaceIface))

typedef struct _RygelLocalSearchResourcesIface RygelLocalSearchResourcesIface;
typedef struct _RygelLocalSearchResourcesIfaceIface RygelLocalSearchResourcesIfaceIface;

#define RYGEL_LOCAL_SEARCH_TYPE_RESOURCES_IFACE_PROXY (rygel_local_search_resources_iface_proxy_get_type ())

#define RYGEL_LOCAL_SEARCH_TYPE_MINER_FILES_INDEX_IFACE (rygel_local_search_miner_files_index_iface_get_type ())
#define RYGEL_LOCAL_SEARCH_MINER_FILES_INDEX_IFACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_LOCAL_SEARCH_TYPE_MINER_FILES_INDEX_IFACE, RygelLocalSearchMinerFilesIndexIface))
#define RYGEL_LOCAL_SEARCH_IS_MINER_FILES_INDEX_IFACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_LOCAL_SEARCH_TYPE_MINER_FILES_INDEX_IFACE))
#define RYGEL_LOCAL_SEARCH_MINER_FILES_INDEX_IFACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_LOCAL_SEARCH_TYPE_MINER_FILES_INDEX_IFACE, RygelLocalSearchMinerFilesIndexIfaceIface))

typedef struct _RygelLocalSearchMinerFilesIndexIface RygelLocalSearchMinerFilesIndexIface;
typedef struct _RygelLocalSearchMinerFilesIndexIfaceIface RygelLocalSearchMinerFilesIndexIfaceIface;

#define RYGEL_LOCAL_SEARCH_TYPE_MINER_FILES_INDEX_IFACE_PROXY (rygel_local_search_miner_files_index_iface_proxy_get_type ())

struct _RygelLocalSearchRootContainer {
	RygelSimpleContainer parent_instance;
	RygelLocalSearchRootContainerPrivate * priv;
};

struct _RygelLocalSearchRootContainerClass {
	RygelSimpleContainerClass parent_class;
};

struct _RygelLocalSearchCategoryContainer {
	RygelSimpleContainer parent_instance;
	RygelLocalSearchCategoryContainerPrivate * priv;
	RygelLocalSearchItemFactory* item_factory;
};

struct _RygelLocalSearchCategoryContainerClass {
	RygelSimpleContainerClass parent_class;
};

struct _RygelLocalSearchMusic {
	RygelLocalSearchCategoryContainer parent_instance;
	RygelLocalSearchMusicPrivate * priv;
};

struct _RygelLocalSearchMusicClass {
	RygelLocalSearchCategoryContainerClass parent_class;
};

struct _RygelLocalSearchVideos {
	RygelLocalSearchCategoryContainer parent_instance;
	RygelLocalSearchVideosPrivate * priv;
};

struct _RygelLocalSearchVideosClass {
	RygelLocalSearchCategoryContainerClass parent_class;
};

struct _RygelLocalSearchPictures {
	RygelLocalSearchCategoryContainer parent_instance;
	RygelLocalSearchPicturesPrivate * priv;
};

struct _RygelLocalSearchPicturesClass {
	RygelLocalSearchCategoryContainerClass parent_class;
};

struct _RygelLocalSearchMetadataContainer {
	RygelSimpleContainer parent_instance;
	RygelLocalSearchMetadataContainerPrivate * priv;
	RygelLocalSearchItemFactory* item_factory;
	RygelLocalSearchQueryTriplets* triplets;
};

struct _RygelLocalSearchMetadataContainerClass {
	RygelSimpleContainerClass parent_class;
	RygelLocalSearchSelectionQuery* (*create_query) (RygelLocalSearchMetadataContainer* self);
	RygelLocalSearchSearchContainer* (*create_container) (RygelLocalSearchMetadataContainer* self, const gchar* id, const gchar* title, const gchar* value);
	gchar* (*create_id_for_title) (RygelLocalSearchMetadataContainer* self, const gchar* title);
	gchar* (*create_title_for_value) (RygelLocalSearchMetadataContainer* self, const gchar* value);
	gchar* (*create_filter) (RygelLocalSearchMetadataContainer* self, const gchar* variable, const gchar* value);
};

struct _RygelLocalSearchMetadataValues {
	RygelLocalSearchMetadataContainer parent_instance;
	RygelLocalSearchMetadataValuesPrivate * priv;
};

struct _RygelLocalSearchMetadataValuesClass {
	RygelLocalSearchMetadataContainerClass parent_class;
};

struct _RygelLocalSearchMetadataMultiValues {
	RygelLocalSearchMetadataContainer parent_instance;
	RygelLocalSearchMetadataMultiValuesPrivate * priv;
	gchar** key_chain;
	gint key_chain_length1;
};

struct _RygelLocalSearchMetadataMultiValuesClass {
	RygelLocalSearchMetadataContainerClass parent_class;
};

struct _RygelLocalSearchTags {
	RygelLocalSearchMetadataMultiValues parent_instance;
	RygelLocalSearchTagsPrivate * priv;
};

struct _RygelLocalSearchTagsClass {
	RygelLocalSearchMetadataMultiValuesClass parent_class;
};

struct _RygelLocalSearchYears {
	RygelLocalSearchMetadataValues parent_instance;
	RygelLocalSearchYearsPrivate * priv;
};

struct _RygelLocalSearchYearsClass {
	RygelLocalSearchMetadataValuesClass parent_class;
};

struct _RygelLocalSearchTitles {
	RygelLocalSearchMetadataValues parent_instance;
	RygelLocalSearchTitlesPrivate * priv;
};

struct _RygelLocalSearchTitlesClass {
	RygelLocalSearchMetadataValuesClass parent_class;
};

struct _RygelLocalSearchAlbums {
	RygelLocalSearchMetadataValues parent_instance;
	RygelLocalSearchAlbumsPrivate * priv;
};

struct _RygelLocalSearchAlbumsClass {
	RygelLocalSearchMetadataValuesClass parent_class;
};

struct _RygelLocalSearchArtists {
	RygelLocalSearchMetadataValues parent_instance;
	RygelLocalSearchArtistsPrivate * priv;
};

struct _RygelLocalSearchArtistsClass {
	RygelLocalSearchMetadataValuesClass parent_class;
};

struct _RygelLocalSearchGenre {
	RygelLocalSearchMetadataValues parent_instance;
	RygelLocalSearchGenrePrivate * priv;
};

struct _RygelLocalSearchGenreClass {
	RygelLocalSearchMetadataValuesClass parent_class;
};

struct _RygelLocalSearchSearchContainer {
	RygelSimpleContainer parent_instance;
	RygelLocalSearchSearchContainerPrivate * priv;
	RygelLocalSearchSelectionQuery* query;
	RygelLocalSearchItemFactory* item_factory;
};

struct _RygelLocalSearchSearchContainerClass {
	RygelSimpleContainerClass parent_class;
};

struct _RygelLocalSearchNew {
	RygelLocalSearchSearchContainer parent_instance;
	RygelLocalSearchNewPrivate * priv;
};

struct _RygelLocalSearchNewClass {
	RygelLocalSearchSearchContainerClass parent_class;
};

struct _RygelLocalSearchCategoryAllContainer {
	RygelLocalSearchSearchContainer parent_instance;
	RygelLocalSearchCategoryAllContainerPrivate * priv;
};

struct _RygelLocalSearchCategoryAllContainerClass {
	RygelLocalSearchSearchContainerClass parent_class;
};

struct _RygelLocalSearchUPnPPropertyMap {
	GObject parent_instance;
	RygelLocalSearchUPnPPropertyMapPrivate * priv;
};

struct _RygelLocalSearchUPnPPropertyMapClass {
	GObjectClass parent_class;
};

struct _RygelLocalSearchQuery {
	GTypeInstance parent_instance;
	volatile int ref_count;
	RygelLocalSearchQueryPrivate * priv;
	RygelLocalSearchQueryTriplets* triplets;
};

struct _RygelLocalSearchQueryClass {
	GTypeClass parent_class;
	void (*finalize) (RygelLocalSearchQuery *self);
	void (*execute) (RygelLocalSearchQuery* self, TrackerSparqlConnection* resources, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*execute_finish) (RygelLocalSearchQuery* self, GAsyncResult* _res_, GError** error);
	gchar* (*to_string) (RygelLocalSearchQuery* self);
};

struct _RygelLocalSearchSelectionQuery {
	RygelLocalSearchQuery parent_instance;
	RygelLocalSearchSelectionQueryPrivate * priv;
	GeeArrayList* variables;
	GeeArrayList* filters;
	gchar* graph;
	gchar* order_by;
	gint offset;
	gint max_count;
	TrackerSparqlCursor* result;
};

struct _RygelLocalSearchSelectionQueryClass {
	RygelLocalSearchQueryClass parent_class;
};

struct _RygelLocalSearchDeletionQuery {
	RygelLocalSearchQuery parent_instance;
	RygelLocalSearchDeletionQueryPrivate * priv;
};

struct _RygelLocalSearchDeletionQueryClass {
	RygelLocalSearchQueryClass parent_class;
};

struct _RygelLocalSearchInsertionQuery {
	RygelLocalSearchQuery parent_instance;
	RygelLocalSearchInsertionQueryPrivate * priv;
	gchar* id;
};

struct _RygelLocalSearchInsertionQueryClass {
	RygelLocalSearchQueryClass parent_class;
};

struct _RygelLocalSearchCleanupQuery {
	RygelLocalSearchQuery parent_instance;
	RygelLocalSearchCleanupQueryPrivate * priv;
};

struct _RygelLocalSearchCleanupQueryClass {
	RygelLocalSearchQueryClass parent_class;
};

struct _RygelLocalSearchQueryTriplet {
	GTypeInstance parent_instance;
	volatile int ref_count;
	RygelLocalSearchQueryTripletPrivate * priv;
	gchar* graph;
	gchar* subject;
	gchar* predicate;
	gchar* obj;
	RygelLocalSearchQueryTriplet* next;
};

struct _RygelLocalSearchQueryTripletClass {
	GTypeClass parent_class;
	void (*finalize) (RygelLocalSearchQueryTriplet *self);
};

struct _RygelLocalSearchQueryTriplets {
	GeeArrayList parent_instance;
	RygelLocalSearchQueryTripletsPrivate * priv;
};

struct _RygelLocalSearchQueryTripletsClass {
	GeeArrayListClass parent_class;
};

struct _RygelLocalSearchItemFactory {
	GTypeInstance parent_instance;
	volatile int ref_count;
	RygelLocalSearchItemFactoryPrivate * priv;
	gchar* category;
	gchar* category_iri;
	gchar* graph;
	gchar* graph_iri;
	gchar* upnp_class;
	gchar* upload_dir;
	GeeArrayList* properties;
};

struct _RygelLocalSearchItemFactoryClass {
	GTypeClass parent_class;
	void (*finalize) (RygelLocalSearchItemFactory *self);
	RygelMediaFileItem* (*create) (RygelLocalSearchItemFactory* self, const gchar* id, const gchar* uri, RygelLocalSearchSearchContainer* parent, TrackerSparqlCursor* metadata, GError** error);
	void (*set_metadata) (RygelLocalSearchItemFactory* self, RygelMediaFileItem* item, const gchar* uri, TrackerSparqlCursor* metadata, GError** error);
	void (*add_resources) (RygelLocalSearchItemFactory* self, RygelMediaFileItem* item, GError** error);
};

struct _RygelLocalSearchVideoItemFactory {
	RygelLocalSearchItemFactory parent_instance;
	RygelLocalSearchVideoItemFactoryPrivate * priv;
};

struct _RygelLocalSearchVideoItemFactoryClass {
	RygelLocalSearchItemFactoryClass parent_class;
};

struct _RygelLocalSearchMusicItemFactory {
	RygelLocalSearchItemFactory parent_instance;
	RygelLocalSearchMusicItemFactoryPrivate * priv;
};

struct _RygelLocalSearchMusicItemFactoryClass {
	RygelLocalSearchItemFactoryClass parent_class;
};

struct _RygelLocalSearchPictureItemFactory {
	RygelLocalSearchItemFactory parent_instance;
	RygelLocalSearchPictureItemFactoryPrivate * priv;
};

struct _RygelLocalSearchPictureItemFactoryClass {
	RygelLocalSearchItemFactoryClass parent_class;
};

struct _RygelLocalSearchPlugin {
	RygelMediaServerPlugin parent_instance;
	RygelLocalSearchPluginPrivate * priv;
};

struct _RygelLocalSearchPluginClass {
	RygelMediaServerPluginClass parent_class;
};

struct _RygelLocalSearchPluginFactory {
	GTypeInstance parent_instance;
	volatile int ref_count;
	RygelLocalSearchPluginFactoryPrivate * priv;
};

struct _RygelLocalSearchPluginFactoryClass {
	GTypeClass parent_class;
	void (*finalize) (RygelLocalSearchPluginFactory *self);
};

struct _Event {
	gint graph_id;
	gint subject_id;
	gint pred_id;
	gint object_id;
};

struct _RygelLocalSearchStatsIfaceIface {
	GTypeInterface parent_iface;
	gchar** (*get) (RygelLocalSearchStatsIface* self, gint* result_length1, gint* result_length2, GError** error);
};

struct _RygelLocalSearchResourcesIfaceIface {
	GTypeInterface parent_iface;
	void (*sparql_query) (RygelLocalSearchResourcesIface* self, const gchar* query, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar** (*sparql_query_finish) (RygelLocalSearchResourcesIface* self, GAsyncResult* _res_, gint* result_length1, gint* result_length2, GError** error);
	void (*sparql_update) (RygelLocalSearchResourcesIface* self, const gchar* query, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*sparql_update_finish) (RygelLocalSearchResourcesIface* self, GAsyncResult* _res_, GError** error);
	void (*sparql_update_blank) (RygelLocalSearchResourcesIface* self, const gchar* query, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GHashTable** (*sparql_update_blank_finish) (RygelLocalSearchResourcesIface* self, GAsyncResult* _res_, gint* result_length1, gint* result_length2, GError** error);
};

struct _RygelLocalSearchMinerFilesIndexIfaceIface {
	GTypeInterface parent_iface;
	void (*index_file) (RygelLocalSearchMinerFilesIndexIface* self, const gchar* uri, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*index_file_finish) (RygelLocalSearchMinerFilesIndexIface* self, GAsyncResult* _res_, GError** error);
};

VALA_EXTERN GType rygel_local_search_root_container_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchRootContainer, g_object_unref)
VALA_EXTERN TrackerSparqlConnection* rygel_local_search_root_container_connection;
VALA_EXTERN gchar* rygel_local_search_root_container_connected_service;
VALA_EXTERN RygelLocalSearchRootContainer* rygel_local_search_root_container_new (const gchar* title,
                                                                      GError** error);
VALA_EXTERN RygelLocalSearchRootContainer* rygel_local_search_root_container_construct (GType object_type,
                                                                            const gchar* title,
                                                                            GError** error);
VALA_EXTERN GType rygel_local_search_category_container_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchCategoryContainer, g_object_unref)
VALA_EXTERN gpointer rygel_local_search_item_factory_ref (gpointer instance);
VALA_EXTERN void rygel_local_search_item_factory_unref (gpointer instance);
VALA_EXTERN GParamSpec* rygel_local_search_param_spec_item_factory (const gchar* name,
                                                        const gchar* nick,
                                                        const gchar* blurb,
                                                        GType object_type,
                                                        GParamFlags flags);
VALA_EXTERN void rygel_local_search_value_set_item_factory (GValue* value,
                                                gpointer v_object);
VALA_EXTERN void rygel_local_search_value_take_item_factory (GValue* value,
                                                 gpointer v_object);
VALA_EXTERN gpointer rygel_local_search_value_get_item_factory (const GValue* value);
VALA_EXTERN GType rygel_local_search_item_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchItemFactory, rygel_local_search_item_factory_unref)
VALA_EXTERN RygelLocalSearchCategoryContainer* rygel_local_search_category_container_construct (GType object_type,
                                                                                    const gchar* id,
                                                                                    RygelMediaContainer* parent,
                                                                                    const gchar* title,
                                                                                    RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN void rygel_local_search_category_container_add_create_class (RygelLocalSearchCategoryContainer* self,
                                                             const gchar* create_class);
VALA_EXTERN GType rygel_local_search_music_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchMusic, g_object_unref)
VALA_EXTERN RygelLocalSearchMusic* rygel_local_search_music_new (const gchar* id,
                                                     RygelMediaContainer* parent,
                                                     const gchar* title);
VALA_EXTERN RygelLocalSearchMusic* rygel_local_search_music_construct (GType object_type,
                                                           const gchar* id,
                                                           RygelMediaContainer* parent,
                                                           const gchar* title);
VALA_EXTERN GType rygel_local_search_videos_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchVideos, g_object_unref)
VALA_EXTERN RygelLocalSearchVideos* rygel_local_search_videos_new (const gchar* id,
                                                       RygelMediaContainer* parent,
                                                       const gchar* title);
VALA_EXTERN RygelLocalSearchVideos* rygel_local_search_videos_construct (GType object_type,
                                                             const gchar* id,
                                                             RygelMediaContainer* parent,
                                                             const gchar* title);
VALA_EXTERN GType rygel_local_search_pictures_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchPictures, g_object_unref)
VALA_EXTERN RygelLocalSearchPictures* rygel_local_search_pictures_new (const gchar* id,
                                                           RygelMediaContainer* parent,
                                                           const gchar* title);
VALA_EXTERN RygelLocalSearchPictures* rygel_local_search_pictures_construct (GType object_type,
                                                                 const gchar* id,
                                                                 RygelMediaContainer* parent,
                                                                 const gchar* title);
VALA_EXTERN GType rygel_local_search_metadata_container_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchMetadataContainer, g_object_unref)
VALA_EXTERN gpointer rygel_local_search_query_ref (gpointer instance);
VALA_EXTERN void rygel_local_search_query_unref (gpointer instance);
VALA_EXTERN GParamSpec* rygel_local_search_param_spec_query (const gchar* name,
                                                 const gchar* nick,
                                                 const gchar* blurb,
                                                 GType object_type,
                                                 GParamFlags flags);
VALA_EXTERN void rygel_local_search_value_set_query (GValue* value,
                                         gpointer v_object);
VALA_EXTERN void rygel_local_search_value_take_query (GValue* value,
                                          gpointer v_object);
VALA_EXTERN gpointer rygel_local_search_value_get_query (const GValue* value);
VALA_EXTERN GType rygel_local_search_query_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchQuery, rygel_local_search_query_unref)
VALA_EXTERN GType rygel_local_search_selection_query_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchSelectionQuery, rygel_local_search_query_unref)
VALA_EXTERN GType rygel_local_search_search_container_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchSearchContainer, g_object_unref)
VALA_EXTERN GType rygel_local_search_query_triplets_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchQueryTriplets, g_object_unref)
VALA_EXTERN GType rygel_local_search_metadata_values_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchMetadataValues, g_object_unref)
VALA_EXTERN RygelLocalSearchMetadataValues* rygel_local_search_metadata_values_construct (GType object_type,
                                                                              const gchar* id,
                                                                              RygelMediaContainer* parent,
                                                                              const gchar* title,
                                                                              RygelLocalSearchItemFactory* item_factory,
                                                                              const gchar* property,
                                                                              const gchar* child_class);
VALA_EXTERN RygelLocalSearchMetadataContainer* rygel_local_search_metadata_container_construct (GType object_type,
                                                                                    const gchar* id,
                                                                                    RygelMediaContainer* parent,
                                                                                    const gchar* title,
                                                                                    RygelLocalSearchItemFactory* item_factory,
                                                                                    const gchar* child_class);
VALA_EXTERN RygelLocalSearchSelectionQuery* rygel_local_search_metadata_container_create_query (RygelLocalSearchMetadataContainer* self);
VALA_EXTERN RygelLocalSearchSearchContainer* rygel_local_search_metadata_container_create_container (RygelLocalSearchMetadataContainer* self,
                                                                                         const gchar* id,
                                                                                         const gchar* title,
                                                                                         const gchar* value);
VALA_EXTERN gchar* rygel_local_search_metadata_container_create_id_for_title (RygelLocalSearchMetadataContainer* self,
                                                                  const gchar* title);
VALA_EXTERN gchar* rygel_local_search_metadata_container_create_title_for_value (RygelLocalSearchMetadataContainer* self,
                                                                     const gchar* value);
VALA_EXTERN gchar* rygel_local_search_metadata_container_create_filter (RygelLocalSearchMetadataContainer* self,
                                                            const gchar* variable,
                                                            const gchar* value);
VALA_EXTERN GType rygel_local_search_metadata_multi_values_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchMetadataMultiValues, g_object_unref)
VALA_EXTERN RygelLocalSearchMetadataMultiValues* rygel_local_search_metadata_multi_values_construct (GType object_type,
                                                                                         const gchar* id,
                                                                                         RygelMediaContainer* parent,
                                                                                         const gchar* title,
                                                                                         RygelLocalSearchItemFactory* item_factory,
                                                                                         gchar** key_chain,
                                                                                         gint key_chain_length1,
                                                                                         const gchar* child_class);
VALA_EXTERN GType rygel_local_search_tags_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchTags, g_object_unref)
VALA_EXTERN RygelLocalSearchTags* rygel_local_search_tags_new (RygelMediaContainer* parent,
                                                   RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN RygelLocalSearchTags* rygel_local_search_tags_construct (GType object_type,
                                                         RygelMediaContainer* parent,
                                                         RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN GType rygel_local_search_years_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchYears, g_object_unref)
VALA_EXTERN RygelLocalSearchYears* rygel_local_search_years_new (RygelMediaContainer* parent,
                                                     RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN RygelLocalSearchYears* rygel_local_search_years_construct (GType object_type,
                                                           RygelMediaContainer* parent,
                                                           RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN GType rygel_local_search_titles_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchTitles, g_object_unref)
VALA_EXTERN RygelLocalSearchTitles* rygel_local_search_titles_new (RygelMediaContainer* parent,
                                                       RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN RygelLocalSearchTitles* rygel_local_search_titles_construct (GType object_type,
                                                             RygelMediaContainer* parent,
                                                             RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN GType rygel_local_search_albums_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchAlbums, g_object_unref)
VALA_EXTERN RygelLocalSearchAlbums* rygel_local_search_albums_new (RygelLocalSearchMusic* parent);
VALA_EXTERN RygelLocalSearchAlbums* rygel_local_search_albums_construct (GType object_type,
                                                             RygelLocalSearchMusic* parent);
VALA_EXTERN GType rygel_local_search_artists_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchArtists, g_object_unref)
VALA_EXTERN RygelLocalSearchArtists* rygel_local_search_artists_new (RygelLocalSearchMusic* parent);
VALA_EXTERN RygelLocalSearchArtists* rygel_local_search_artists_construct (GType object_type,
                                                               RygelLocalSearchMusic* parent);
VALA_EXTERN GType rygel_local_search_genre_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchGenre, g_object_unref)
VALA_EXTERN RygelLocalSearchGenre* rygel_local_search_genre_new (RygelLocalSearchMusic* parent);
VALA_EXTERN RygelLocalSearchGenre* rygel_local_search_genre_construct (GType object_type,
                                                           RygelLocalSearchMusic* parent);
VALA_EXTERN GType rygel_local_search_new_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchNew, g_object_unref)
VALA_EXTERN RygelLocalSearchNew* rygel_local_search_new_new (RygelMediaContainer* parent,
                                                 RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN RygelLocalSearchNew* rygel_local_search_new_construct (GType object_type,
                                                       RygelMediaContainer* parent,
                                                       RygelLocalSearchItemFactory* item_factory);
VALA_EXTERN RygelLocalSearchSearchContainer* rygel_local_search_search_container_new (const gchar* id,
                                                                          RygelMediaContainer* parent,
                                                                          const gchar* title,
                                                                          RygelLocalSearchItemFactory* item_factory,
                                                                          RygelLocalSearchQueryTriplets* triplets,
                                                                          GeeArrayList* filters);
VALA_EXTERN RygelLocalSearchSearchContainer* rygel_local_search_search_container_construct (GType object_type,
                                                                                const gchar* id,
                                                                                RygelMediaContainer* parent,
                                                                                const gchar* title,
                                                                                RygelLocalSearchItemFactory* item_factory,
                                                                                RygelLocalSearchQueryTriplets* triplets,
                                                                                GeeArrayList* filters);
VALA_EXTERN void rygel_local_search_search_container_execute_query (RygelLocalSearchSearchContainer* self,
                                                        RygelSearchExpression* expression,
                                                        const gchar* sort_criteria,
                                                        guint offset,
                                                        guint max_count,
                                                        GCancellable* cancellable,
                                                        GAsyncReadyCallback _callback_,
                                                        gpointer _user_data_);
VALA_EXTERN RygelMediaObjects* rygel_local_search_search_container_execute_query_finish (RygelLocalSearchSearchContainer* self,
                                                                             GAsyncResult* _res_,
                                                                             guint* total_matches,
                                                                             GError** error);
VALA_EXTERN gchar* rygel_local_search_search_container_create_child_id_for_urn (RygelLocalSearchSearchContainer* self,
                                                                    const gchar* urn);
VALA_EXTERN gchar* rygel_local_search_search_container_get_item_info (RygelLocalSearchSearchContainer* self,
                                                          const gchar* item_id,
                                                          gchar** parent_id);
VALA_EXTERN GType rygel_local_search_category_all_container_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchCategoryAllContainer, g_object_unref)
VALA_EXTERN RygelLocalSearchCategoryAllContainer* rygel_local_search_category_all_container_new (RygelLocalSearchCategoryContainer* parent);
VALA_EXTERN RygelLocalSearchCategoryAllContainer* rygel_local_search_category_all_container_construct (GType object_type,
                                                                                           RygelLocalSearchCategoryContainer* parent);
VALA_EXTERN GType rygel_local_search_upn_pproperty_map_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchUPnPPropertyMap, g_object_unref)
VALA_EXTERN RygelLocalSearchUPnPPropertyMap* rygel_local_search_upn_pproperty_map_get_property_map (void);
VALA_EXTERN gchar* rygel_local_search_upn_pproperty_map_get (RygelLocalSearchUPnPPropertyMap* self,
                                                 const gchar* property);
VALA_EXTERN RygelLocalSearchQuery* rygel_local_search_query_construct (GType object_type,
                                                           RygelLocalSearchQueryTriplets* triplets);
VALA_EXTERN void rygel_local_search_query_execute (RygelLocalSearchQuery* self,
                                       TrackerSparqlConnection* resources,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN void rygel_local_search_query_execute_finish (RygelLocalSearchQuery* self,
                                              GAsyncResult* _res_,
                                              GError** error);
VALA_EXTERN gchar* rygel_local_search_query_to_string (RygelLocalSearchQuery* self);
VALA_EXTERN gchar* rygel_local_search_query_escape_regex (const gchar* literal);
VALA_EXTERN gchar* rygel_local_search_query_escape_string (const gchar* literal);
VALA_EXTERN RygelLocalSearchSelectionQuery* rygel_local_search_selection_query_new (GeeArrayList* variables,
                                                                        RygelLocalSearchQueryTriplets* triplets,
                                                                        GeeArrayList* filters,
                                                                        const gchar* graph,
                                                                        const gchar* order_by,
                                                                        gint offset,
                                                                        gint max_count);
VALA_EXTERN RygelLocalSearchSelectionQuery* rygel_local_search_selection_query_construct (GType object_type,
                                                                              GeeArrayList* variables,
                                                                              RygelLocalSearchQueryTriplets* triplets,
                                                                              GeeArrayList* filters,
                                                                              const gchar* graph,
                                                                              const gchar* order_by,
                                                                              gint offset,
                                                                              gint max_count);
VALA_EXTERN RygelLocalSearchSelectionQuery* rygel_local_search_selection_query_new_clone (RygelLocalSearchSelectionQuery* query);
VALA_EXTERN RygelLocalSearchSelectionQuery* rygel_local_search_selection_query_construct_clone (GType object_type,
                                                                                    RygelLocalSearchSelectionQuery* query);
VALA_EXTERN GType rygel_local_search_deletion_query_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchDeletionQuery, rygel_local_search_query_unref)
VALA_EXTERN RygelLocalSearchDeletionQuery* rygel_local_search_deletion_query_new (const gchar* id);
VALA_EXTERN RygelLocalSearchDeletionQuery* rygel_local_search_deletion_query_construct (GType object_type,
                                                                            const gchar* id);
VALA_EXTERN GType rygel_local_search_insertion_query_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchInsertionQuery, rygel_local_search_query_unref)
VALA_EXTERN RygelLocalSearchInsertionQuery* rygel_local_search_insertion_query_new (RygelMediaFileItem* item,
                                                                        const gchar* category,
                                                                        const gchar* graph);
VALA_EXTERN RygelLocalSearchInsertionQuery* rygel_local_search_insertion_query_construct (GType object_type,
                                                                              RygelMediaFileItem* item,
                                                                              const gchar* category,
                                                                              const gchar* graph);
VALA_EXTERN GType rygel_local_search_cleanup_query_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchCleanupQuery, rygel_local_search_query_unref)
VALA_EXTERN RygelLocalSearchCleanupQuery* rygel_local_search_cleanup_query_new (const gchar* category,
                                                                    const gchar* graph);
VALA_EXTERN RygelLocalSearchCleanupQuery* rygel_local_search_cleanup_query_construct (GType object_type,
                                                                          const gchar* category,
                                                                          const gchar* graph);
VALA_EXTERN gpointer rygel_local_search_query_triplet_ref (gpointer instance);
VALA_EXTERN void rygel_local_search_query_triplet_unref (gpointer instance);
VALA_EXTERN GParamSpec* rygel_local_search_param_spec_query_triplet (const gchar* name,
                                                         const gchar* nick,
                                                         const gchar* blurb,
                                                         GType object_type,
                                                         GParamFlags flags);
VALA_EXTERN void rygel_local_search_value_set_query_triplet (GValue* value,
                                                 gpointer v_object);
VALA_EXTERN void rygel_local_search_value_take_query_triplet (GValue* value,
                                                  gpointer v_object);
VALA_EXTERN gpointer rygel_local_search_value_get_query_triplet (const GValue* value);
VALA_EXTERN GType rygel_local_search_query_triplet_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchQueryTriplet, rygel_local_search_query_triplet_unref)
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_new (const gchar* subject,
                                                                    const gchar* predicate,
                                                                    const gchar* obj);
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_construct (GType object_type,
                                                                          const gchar* subject,
                                                                          const gchar* predicate,
                                                                          const gchar* obj);
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_new_with_graph (const gchar* graph,
                                                                               const gchar* subject,
                                                                               const gchar* predicate,
                                                                               const gchar* object);
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_construct_with_graph (GType object_type,
                                                                                     const gchar* graph,
                                                                                     const gchar* subject,
                                                                                     const gchar* predicate,
                                                                                     const gchar* object);
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_new_chain (const gchar* subject,
                                                                          const gchar* predicate,
                                                                          RygelLocalSearchQueryTriplet* next);
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_construct_chain (GType object_type,
                                                                                const gchar* subject,
                                                                                const gchar* predicate,
                                                                                RygelLocalSearchQueryTriplet* next);
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_new_clone (RygelLocalSearchQueryTriplet* triplet);
VALA_EXTERN RygelLocalSearchQueryTriplet* rygel_local_search_query_triplet_construct_clone (GType object_type,
                                                                                RygelLocalSearchQueryTriplet* triplet);
VALA_EXTERN gboolean rygel_local_search_query_triplet_equal_func (RygelLocalSearchQueryTriplet* a,
                                                      RygelLocalSearchQueryTriplet* b);
VALA_EXTERN gchar* rygel_local_search_query_triplet_to_string (RygelLocalSearchQueryTriplet* self,
                                                   gboolean include_subject);
VALA_EXTERN RygelLocalSearchQueryTriplets* rygel_local_search_query_triplets_new (void);
VALA_EXTERN RygelLocalSearchQueryTriplets* rygel_local_search_query_triplets_construct (GType object_type);
VALA_EXTERN RygelLocalSearchQueryTriplets* rygel_local_search_query_triplets_new_clone (RygelLocalSearchQueryTriplets* triplets);
VALA_EXTERN RygelLocalSearchQueryTriplets* rygel_local_search_query_triplets_construct_clone (GType object_type,
                                                                                  RygelLocalSearchQueryTriplets* triplets);
VALA_EXTERN gchar* rygel_local_search_query_triplets_serialize (RygelLocalSearchQueryTriplets* self);
VALA_EXTERN GType rygel_local_search_item_factory_metadata_get_type (void) G_GNUC_CONST ;
VALA_EXTERN RygelLocalSearchItemFactory* rygel_local_search_item_factory_construct (GType object_type,
                                                                        const gchar* category,
                                                                        const gchar* category_iri,
                                                                        const gchar* graph,
                                                                        const gchar* graph_iri,
                                                                        const gchar* upnp_class,
                                                                        const gchar* upload_dir);
VALA_EXTERN RygelMediaFileItem* rygel_local_search_item_factory_create (RygelLocalSearchItemFactory* self,
                                                            const gchar* id,
                                                            const gchar* uri,
                                                            RygelLocalSearchSearchContainer* parent,
                                                            TrackerSparqlCursor* metadata,
                                                            GError** error);
VALA_EXTERN void rygel_local_search_item_factory_set_ref_id (RygelLocalSearchItemFactory* self,
                                                 RygelMediaFileItem* item,
                                                 const gchar* prefix);
VALA_EXTERN void rygel_local_search_item_factory_set_metadata (RygelLocalSearchItemFactory* self,
                                                   RygelMediaFileItem* item,
                                                   const gchar* uri,
                                                   TrackerSparqlCursor* metadata,
                                                   GError** error);
VALA_EXTERN void rygel_local_search_item_factory_add_resources (RygelLocalSearchItemFactory* self,
                                                    RygelMediaFileItem* item,
                                                    GError** error);
VALA_EXTERN GType rygel_local_search_video_item_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchVideoItemFactory, rygel_local_search_item_factory_unref)
VALA_EXTERN RygelLocalSearchVideoItemFactory* rygel_local_search_video_item_factory_new (void);
VALA_EXTERN RygelLocalSearchVideoItemFactory* rygel_local_search_video_item_factory_construct (GType object_type);
VALA_EXTERN GType rygel_local_search_music_item_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchMusicItemFactory, rygel_local_search_item_factory_unref)
VALA_EXTERN RygelLocalSearchMusicItemFactory* rygel_local_search_music_item_factory_new (void);
VALA_EXTERN RygelLocalSearchMusicItemFactory* rygel_local_search_music_item_factory_construct (GType object_type);
VALA_EXTERN GType rygel_local_search_picture_item_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchPictureItemFactory, rygel_local_search_item_factory_unref)
VALA_EXTERN RygelLocalSearchPictureItemFactory* rygel_local_search_picture_item_factory_new (void);
VALA_EXTERN RygelLocalSearchPictureItemFactory* rygel_local_search_picture_item_factory_construct (GType object_type);
VALA_EXTERN GType rygel_local_search_plugin_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchPlugin, g_object_unref)
VALA_EXTERN RygelLocalSearchPlugin* rygel_local_search_plugin_new (GError** error);
VALA_EXTERN RygelLocalSearchPlugin* rygel_local_search_plugin_construct (GType object_type,
                                                             GError** error);
VALA_EXTERN void module_init (RygelPluginLoader* loader);
VALA_EXTERN gpointer rygel_local_search_plugin_factory_ref (gpointer instance);
VALA_EXTERN void rygel_local_search_plugin_factory_unref (gpointer instance);
VALA_EXTERN GParamSpec* rygel_local_search_param_spec_plugin_factory (const gchar* name,
                                                          const gchar* nick,
                                                          const gchar* blurb,
                                                          GType object_type,
                                                          GParamFlags flags);
VALA_EXTERN void rygel_local_search_value_set_plugin_factory (GValue* value,
                                                  gpointer v_object);
VALA_EXTERN void rygel_local_search_value_take_plugin_factory (GValue* value,
                                                   gpointer v_object);
VALA_EXTERN gpointer rygel_local_search_value_get_plugin_factory (const GValue* value);
VALA_EXTERN GType rygel_local_search_plugin_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchPluginFactory, rygel_local_search_plugin_factory_unref)
VALA_EXTERN RygelLocalSearchPluginFactory* rygel_local_search_plugin_factory_new (RygelPluginLoader* loader,
                                                                      GError** error);
VALA_EXTERN RygelLocalSearchPluginFactory* rygel_local_search_plugin_factory_construct (GType object_type,
                                                                            RygelPluginLoader* loader,
                                                                            GError** error);
VALA_EXTERN GType event_get_type (void) G_GNUC_CONST ;
VALA_EXTERN Event* event_dup (const Event* self);
VALA_EXTERN void event_free (Event* self);
VALA_EXTERN GType rygel_local_search_stats_iface_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint rygel_local_search_stats_iface_register_object (void* object,
                                                      GDBusConnection* connection,
                                                      const gchar* path,
                                                      GError** error);
VALA_EXTERN GType rygel_local_search_stats_iface_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchStatsIface, g_object_unref)
VALA_EXTERN gchar** rygel_local_search_stats_iface_get (RygelLocalSearchStatsIface* self,
                                            gint* result_length1,
                                            gint* result_length2,
                                            GError** error);
VALA_EXTERN GType rygel_local_search_resources_iface_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint rygel_local_search_resources_iface_register_object (void* object,
                                                          GDBusConnection* connection,
                                                          const gchar* path,
                                                          GError** error);
VALA_EXTERN GType rygel_local_search_resources_iface_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchResourcesIface, g_object_unref)
VALA_EXTERN void rygel_local_search_resources_iface_sparql_query (RygelLocalSearchResourcesIface* self,
                                                      const gchar* query,
                                                      GAsyncReadyCallback _callback_,
                                                      gpointer _user_data_);
VALA_EXTERN gchar** rygel_local_search_resources_iface_sparql_query_finish (RygelLocalSearchResourcesIface* self,
                                                                GAsyncResult* _res_,
                                                                gint* result_length1,
                                                                gint* result_length2,
                                                                GError** error);
VALA_EXTERN void rygel_local_search_resources_iface_sparql_update (RygelLocalSearchResourcesIface* self,
                                                       const gchar* query,
                                                       GAsyncReadyCallback _callback_,
                                                       gpointer _user_data_);
VALA_EXTERN void rygel_local_search_resources_iface_sparql_update_finish (RygelLocalSearchResourcesIface* self,
                                                              GAsyncResult* _res_,
                                                              GError** error);
VALA_EXTERN void rygel_local_search_resources_iface_sparql_update_blank (RygelLocalSearchResourcesIface* self,
                                                             const gchar* query,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
VALA_EXTERN GHashTable** rygel_local_search_resources_iface_sparql_update_blank_finish (RygelLocalSearchResourcesIface* self,
                                                                            GAsyncResult* _res_,
                                                                            gint* result_length1,
                                                                            gint* result_length2,
                                                                            GError** error);
VALA_EXTERN GType rygel_local_search_miner_files_index_iface_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint rygel_local_search_miner_files_index_iface_register_object (void* object,
                                                                  GDBusConnection* connection,
                                                                  const gchar* path,
                                                                  GError** error);
VALA_EXTERN GType rygel_local_search_miner_files_index_iface_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelLocalSearchMinerFilesIndexIface, g_object_unref)
VALA_EXTERN void rygel_local_search_miner_files_index_iface_index_file (RygelLocalSearchMinerFilesIndexIface* self,
                                                            const gchar* uri,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_);
VALA_EXTERN void rygel_local_search_miner_files_index_iface_index_file_finish (RygelLocalSearchMinerFilesIndexIface* self,
                                                                   GAsyncResult* _res_,
                                                                   GError** error);

G_END_DECLS

#endif
