/* rygel-gst-launch-video-item.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-gst-launch-video-item.vala, do not modify */

/*
 * Copyright (C) 2009 Thijs Vermeir <thijsvermeir@gmail.com>
 * Copyright (C) 2010 Nokia Corporation.
 * Copyright (C) 2012 Intel Corporation.
 *
 * Author: Thijs Vermeir <thijsvermeir@gmail.com>
 * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
 *                               <zeeshan.ali@nokia.com>
 *         Jens Georg <jensg@openismus.com>
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "rygel-gst-launch.h"
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "rygel-server.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	RYGEL_GST_LAUNCH_VIDEO_ITEM_0_PROPERTY,
	RYGEL_GST_LAUNCH_VIDEO_ITEM_NUM_PROPERTIES
};
static GParamSpec* rygel_gst_launch_video_item_properties[RYGEL_GST_LAUNCH_VIDEO_ITEM_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))

static gpointer rygel_gst_launch_video_item_parent_class = NULL;

static GType rygel_gst_launch_video_item_get_type_once (void);

RygelGstLaunchVideoItem*
rygel_gst_launch_video_item_construct (GType object_type,
                                       const gchar* id,
                                       RygelMediaContainer* parent,
                                       const gchar* title,
                                       const gchar* mime_type,
                                       const gchar* launch_line)
{
	RygelGstLaunchVideoItem * self = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (parent != NULL, NULL);
	g_return_val_if_fail (title != NULL, NULL);
	g_return_val_if_fail (mime_type != NULL, NULL);
	g_return_val_if_fail (launch_line != NULL, NULL);
	self = (RygelGstLaunchVideoItem*) rygel_video_item_construct (object_type, id, parent, title, RYGEL_VIDEO_ITEM_UPNP_CLASS);
	rygel_media_file_item_set_mime_type ((RygelMediaFileItem*) self, mime_type);
	_tmp0_ = g_uri_escape_string (launch_line, NULL, TRUE);
	_tmp1_ = _tmp0_;
	_tmp2_ = g_strconcat ("gst-launch://", _tmp1_, NULL);
	_tmp3_ = _tmp2_;
	rygel_media_object_add_uri ((RygelMediaObject*) self, _tmp3_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp1_);
	return self;
}

RygelGstLaunchVideoItem*
rygel_gst_launch_video_item_new (const gchar* id,
                                 RygelMediaContainer* parent,
                                 const gchar* title,
                                 const gchar* mime_type,
                                 const gchar* launch_line)
{
	return rygel_gst_launch_video_item_construct (RYGEL_GST_LAUNCH_TYPE_VIDEO_ITEM, id, parent, title, mime_type, launch_line);
}

static void
rygel_gst_launch_video_item_class_init (RygelGstLaunchVideoItemClass * klass,
                                        gpointer klass_data)
{
	rygel_gst_launch_video_item_parent_class = g_type_class_peek_parent (klass);
}

static void
rygel_gst_launch_video_item_instance_init (RygelGstLaunchVideoItem * self,
                                           gpointer klass)
{
}

/**
 * Video item that serves data from a gst-launch commandline.
 */
 G_GNUC_NO_INLINE static GType
rygel_gst_launch_video_item_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelGstLaunchVideoItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_gst_launch_video_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RygelGstLaunchVideoItem), 0, (GInstanceInitFunc) rygel_gst_launch_video_item_instance_init, NULL };
	GType rygel_gst_launch_video_item_type_id;
	rygel_gst_launch_video_item_type_id = g_type_register_static (RYGEL_TYPE_VIDEO_ITEM, "RygelGstLaunchVideoItem", &g_define_type_info, 0);
	return rygel_gst_launch_video_item_type_id;
}

GType
rygel_gst_launch_video_item_get_type (void)
{
	static volatile gsize rygel_gst_launch_video_item_type_id__once = 0;
	if (g_once_init_enter (&rygel_gst_launch_video_item_type_id__once)) {
		GType rygel_gst_launch_video_item_type_id;
		rygel_gst_launch_video_item_type_id = rygel_gst_launch_video_item_get_type_once ();
		g_once_init_leave (&rygel_gst_launch_video_item_type_id__once, rygel_gst_launch_video_item_type_id);
	}
	return rygel_gst_launch_video_item_type_id__once;
}

