/* rygel-media-export-node-query-container.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-media-export-node-query-container.vala, do not modify */

/*
 * Copyright (C) 2011 Jens Georg <mail@jensge.org>.
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "rygel-media-export.h"
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "rygel-server.h"
#include <gio/gio.h>
#include <gee.h>
#include <glib/gi18n-lib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER (rygel_media_export_query_container_get_type ())
#define RYGEL_MEDIA_EXPORT_QUERY_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER, RygelMediaExportQueryContainer))
#define RYGEL_MEDIA_EXPORT_QUERY_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER, RygelMediaExportQueryContainerClass))
#define RYGEL_MEDIA_EXPORT_IS_QUERY_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER))
#define RYGEL_MEDIA_EXPORT_IS_QUERY_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER))
#define RYGEL_MEDIA_EXPORT_QUERY_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER, RygelMediaExportQueryContainerClass))

typedef struct _RygelMediaExportQueryContainer RygelMediaExportQueryContainer;
typedef struct _RygelMediaExportQueryContainerClass RygelMediaExportQueryContainerClass;
typedef struct _RygelMediaExportQueryContainerPrivate RygelMediaExportQueryContainerPrivate;

#define RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER (rygel_media_export_node_query_container_get_type ())
#define RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER, RygelMediaExportNodeQueryContainer))
#define RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER, RygelMediaExportNodeQueryContainerClass))
#define RYGEL_MEDIA_EXPORT_IS_NODE_QUERY_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER))
#define RYGEL_MEDIA_EXPORT_IS_NODE_QUERY_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER))
#define RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER, RygelMediaExportNodeQueryContainerClass))

typedef struct _RygelMediaExportNodeQueryContainer RygelMediaExportNodeQueryContainer;
typedef struct _RygelMediaExportNodeQueryContainerClass RygelMediaExportNodeQueryContainerClass;
typedef struct _RygelMediaExportNodeQueryContainerPrivate RygelMediaExportNodeQueryContainerPrivate;
enum  {
	RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_0_PROPERTY,
	RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_TEMPLATE_PROPERTY,
	RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_ATTRIBUTE_PROPERTY,
	RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_NUM_PROPERTIES
};
static GParamSpec* rygel_media_export_node_query_container_properties[RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER_FACTORY (rygel_media_export_query_container_factory_get_type ())
#define RYGEL_MEDIA_EXPORT_QUERY_CONTAINER_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER_FACTORY, RygelMediaExportQueryContainerFactory))
#define RYGEL_MEDIA_EXPORT_QUERY_CONTAINER_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER_FACTORY, RygelMediaExportQueryContainerFactoryClass))
#define RYGEL_MEDIA_EXPORT_IS_QUERY_CONTAINER_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER_FACTORY))
#define RYGEL_MEDIA_EXPORT_IS_QUERY_CONTAINER_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER_FACTORY))
#define RYGEL_MEDIA_EXPORT_QUERY_CONTAINER_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER_FACTORY, RygelMediaExportQueryContainerFactoryClass))

typedef struct _RygelMediaExportQueryContainerFactory RygelMediaExportQueryContainerFactory;
typedef struct _RygelMediaExportQueryContainerFactoryClass RygelMediaExportQueryContainerFactoryClass;
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))
typedef struct _RygelMediaExportNodeQueryContainerGetChildrenData RygelMediaExportNodeQueryContainerGetChildrenData;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

struct _RygelMediaExportQueryContainer {
	RygelMediaExportDBContainer parent_instance;
	RygelMediaExportQueryContainerPrivate * priv;
};

struct _RygelMediaExportQueryContainerClass {
	RygelMediaExportDBContainerClass parent_class;
};

struct _RygelMediaExportNodeQueryContainer {
	RygelMediaExportQueryContainer parent_instance;
	RygelMediaExportNodeQueryContainerPrivate * priv;
};

struct _RygelMediaExportNodeQueryContainerClass {
	RygelMediaExportQueryContainerClass parent_class;
};

struct _RygelMediaExportNodeQueryContainerPrivate {
	gchar* _template;
	gchar* _attribute;
};

struct _RygelMediaExportNodeQueryContainerGetChildrenData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	RygelMediaExportNodeQueryContainer* self;
	guint offset;
	guint max_count;
	gchar* sort_criteria;
	GCancellable* cancellable;
	RygelMediaObjects* result;
	RygelMediaObjects* children;
	RygelMediaObjects* _tmp0_;
	RygelMediaExportQueryContainerFactory* factory;
	RygelMediaExportQueryContainerFactory* _tmp1_;
	GeeList* data;
	RygelMediaExportMediaCache* _tmp2_;
	const gchar* _tmp3_;
	RygelSearchExpression* _tmp4_;
	RygelSearchExpression* _tmp5_;
	GeeList* _tmp6_;
	GeeList* _meta_data_list;
	gint _meta_data_size;
	GeeList* _tmp7_;
	gint _tmp8_;
	gint _tmp9_;
	gint _meta_data_index;
	gint _tmp10_;
	gint _tmp11_;
	gchar* meta_data;
	GeeList* _tmp12_;
	gpointer _tmp13_;
	gchar* id;
	RygelMediaContainer* container;
	const gchar* _tmp14_;
	const gchar* _tmp15_;
	gchar* _tmp16_;
	const gchar* _tmp17_;
	RygelMediaExportQueryContainer* _tmp18_;
	const gchar* _tmp19_;
	gchar* _tmp20_;
	const gchar* _tmp21_;
	const gchar* _tmp22_;
	gchar* _tmp23_;
	const gchar* _tmp24_;
	const gchar* _tmp25_;
	RygelMediaExportQueryContainer* _tmp26_;
	RygelMediaContainer* _tmp27_;
	RygelMediaContainer* _tmp28_;
	GError* _inner_error0_;
};

static gint RygelMediaExportNodeQueryContainer_private_offset;
static gpointer rygel_media_export_node_query_container_parent_class = NULL;

VALA_EXTERN GType rygel_media_export_query_container_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType rygel_media_export_node_query_container_get_type (void) G_GNUC_CONST ;
VALA_EXTERN RygelMediaExportNodeQueryContainer* rygel_media_export_node_query_container_new (RygelSearchExpression* expression,
                                                                                 const gchar* id,
                                                                                 const gchar* name,
                                                                                 const gchar* template,
                                                                                 const gchar* attribute);
VALA_EXTERN RygelMediaExportNodeQueryContainer* rygel_media_export_node_query_container_construct (GType object_type,
                                                                                       RygelSearchExpression* expression,
                                                                                       const gchar* id,
                                                                                       const gchar* name,
                                                                                       const gchar* template,
                                                                                       const gchar* attribute);
static void rygel_media_export_node_query_container_real_get_children_data_free (gpointer _data);
static void rygel_media_export_node_query_container_real_get_children (RygelMediaContainer* base,
                                                                guint offset,
                                                                guint max_count,
                                                                const gchar* sort_criteria,
                                                                GCancellable* cancellable,
                                                                GAsyncReadyCallback _callback_,
                                                                gpointer _user_data_);
static gboolean rygel_media_export_node_query_container_real_get_children_co (RygelMediaExportNodeQueryContainerGetChildrenData* _data_);
VALA_EXTERN GType rygel_media_export_query_container_factory_get_type (void) G_GNUC_CONST ;
VALA_EXTERN RygelMediaExportQueryContainerFactory* rygel_media_export_query_container_factory_get_default (void);
static const gchar* rygel_media_export_node_query_container_get_attribute (RygelMediaExportNodeQueryContainer* self);
VALA_EXTERN RygelSearchExpression* rygel_media_export_query_container_get_expression (RygelMediaExportQueryContainer* self);
static gboolean rygel_media_export_node_query_container_add_all_container (RygelMediaExportNodeQueryContainer* self);
static const gchar* rygel_media_export_node_query_container_get_template (RygelMediaExportNodeQueryContainer* self);
VALA_EXTERN RygelMediaExportQueryContainer* rygel_media_export_query_container_factory_create_from_description_id (RygelMediaExportQueryContainerFactory* self,
                                                                                                       const gchar* definition_id,
                                                                                                       const gchar* name);
static gint rygel_media_export_node_query_container_real_count_children (RygelMediaExportDBContainer* base);
static void rygel_media_export_node_query_container_set_template (RygelMediaExportNodeQueryContainer* self,
                                                           const gchar* value);
static void rygel_media_export_node_query_container_set_attribute (RygelMediaExportNodeQueryContainer* self,
                                                            const gchar* value);
static void rygel_media_export_node_query_container_finalize (GObject * obj);
static GType rygel_media_export_node_query_container_get_type_once (void);
static void _vala_rygel_media_export_node_query_container_get_property (GObject * object,
                                                                 guint property_id,
                                                                 GValue * value,
                                                                 GParamSpec * pspec);
static void _vala_rygel_media_export_node_query_container_set_property (GObject * object,
                                                                 guint property_id,
                                                                 const GValue * value,
                                                                 GParamSpec * pspec);

static inline gpointer
rygel_media_export_node_query_container_get_instance_private (RygelMediaExportNodeQueryContainer* self)
{
	return G_STRUCT_MEMBER_P (self, RygelMediaExportNodeQueryContainer_private_offset);
}

RygelMediaExportNodeQueryContainer*
rygel_media_export_node_query_container_construct (GType object_type,
                                                   RygelSearchExpression* expression,
                                                   const gchar* id,
                                                   const gchar* name,
                                                   const gchar* template,
                                                   const gchar* attribute)
{
	RygelMediaExportNodeQueryContainer * self = NULL;
	g_return_val_if_fail (expression != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (template != NULL, NULL);
	g_return_val_if_fail (attribute != NULL, NULL);
	self = (RygelMediaExportNodeQueryContainer*) g_object_new (object_type, "id", id, "title", name, "parent", NULL, "child-count", 0, "expression", expression, "template", template, "attribute", attribute, NULL);
	return self;
}

RygelMediaExportNodeQueryContainer*
rygel_media_export_node_query_container_new (RygelSearchExpression* expression,
                                             const gchar* id,
                                             const gchar* name,
                                             const gchar* template,
                                             const gchar* attribute)
{
	return rygel_media_export_node_query_container_construct (RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER, expression, id, name, template, attribute);
}

static void
rygel_media_export_node_query_container_real_get_children_data_free (gpointer _data)
{
	RygelMediaExportNodeQueryContainerGetChildrenData* _data_;
	_data_ = _data;
	_g_free0 (_data_->sort_criteria);
	_g_object_unref0 (_data_->cancellable);
	_g_object_unref0 (_data_->result);
	_g_object_unref0 (_data_->self);
	g_slice_free (RygelMediaExportNodeQueryContainerGetChildrenData, _data_);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
rygel_media_export_node_query_container_real_get_children (RygelMediaContainer* base,
                                                           guint offset,
                                                           guint max_count,
                                                           const gchar* sort_criteria,
                                                           GCancellable* cancellable,
                                                           GAsyncReadyCallback _callback_,
                                                           gpointer _user_data_)
{
	RygelMediaExportNodeQueryContainer * self;
	RygelMediaExportNodeQueryContainerGetChildrenData* _data_;
	RygelMediaExportNodeQueryContainer* _tmp0_;
	gchar* _tmp1_;
	GCancellable* _tmp2_;
	g_return_if_fail (sort_criteria != NULL);
	self = (RygelMediaExportNodeQueryContainer*) base;
	_data_ = g_slice_new0 (RygelMediaExportNodeQueryContainerGetChildrenData);
	_data_->_async_result = g_task_new (G_OBJECT (self), cancellable, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, rygel_media_export_node_query_container_real_get_children_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_data_->offset = offset;
	_data_->max_count = max_count;
	_tmp1_ = g_strdup (sort_criteria);
	_g_free0 (_data_->sort_criteria);
	_data_->sort_criteria = _tmp1_;
	_tmp2_ = _g_object_ref0 (cancellable);
	_g_object_unref0 (_data_->cancellable);
	_data_->cancellable = _tmp2_;
	rygel_media_export_node_query_container_real_get_children_co (_data_);
}

static RygelMediaObjects*
rygel_media_export_node_query_container_get_children_finish (RygelMediaContainer* base,
                                                             GAsyncResult* _res_,
                                                             GError** error)
{
	RygelMediaObjects* result;
	RygelMediaExportNodeQueryContainerGetChildrenData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), error);
	if (NULL == _data_) {
		return NULL;
	}
	result = _data_->result;
	_data_->result = NULL;
	return result;
}

static gchar*
string_replace (const gchar* self,
                const gchar* old,
                const gchar* replacement)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	GError* _inner_error0_ = NULL;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	if ((*((gchar*) self)) == '\0') {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = (*((gchar*) old)) == '\0';
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = g_strcmp0 (old, replacement) == 0;
	}
	if (_tmp0_) {
		gchar* _tmp2_;
		_tmp2_ = g_strdup (self);
		result = _tmp2_;
		return result;
	}
	{
		GRegex* regex = NULL;
		gchar* _tmp3_;
		gchar* _tmp4_;
		GRegex* _tmp5_;
		GRegex* _tmp6_;
		gchar* _tmp7_ = NULL;
		GRegex* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		_tmp3_ = g_regex_escape_string (old, -1);
		_tmp4_ = _tmp3_;
		_tmp5_ = g_regex_new (_tmp4_, 0, 0, &_inner_error0_);
		_tmp6_ = _tmp5_;
		_g_free0 (_tmp4_);
		regex = _tmp6_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == G_REGEX_ERROR) {
				goto __catch0_g_regex_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
		_tmp8_ = regex;
		_tmp9_ = g_regex_replace_literal (_tmp8_, self, (gssize) -1, 0, replacement, 0, &_inner_error0_);
		_tmp7_ = _tmp9_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			_g_regex_unref0 (regex);
			if (_inner_error0_->domain == G_REGEX_ERROR) {
				goto __catch0_g_regex_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
		_tmp10_ = _tmp7_;
		_tmp7_ = NULL;
		result = _tmp10_;
		_g_free0 (_tmp7_);
		_g_regex_unref0 (regex);
		return result;
	}
	goto __finally0;
	__catch0_g_regex_error:
	{
		g_clear_error (&_inner_error0_);
		g_assert_not_reached ();
	}
	__finally0:
	g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
	g_clear_error (&_inner_error0_);
	return NULL;
}

static gboolean
rygel_media_export_node_query_container_real_get_children_co (RygelMediaExportNodeQueryContainerGetChildrenData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = rygel_media_objects_new ();
	_data_->children = _data_->_tmp0_;
	_data_->_tmp1_ = rygel_media_export_query_container_factory_get_default ();
	_data_->factory = _data_->_tmp1_;
	_data_->_tmp2_ = ((RygelMediaExportDBContainer*) _data_->self)->media_db;
	_data_->_tmp3_ = _data_->self->priv->_attribute;
	_data_->_tmp4_ = rygel_media_export_query_container_get_expression ((RygelMediaExportQueryContainer*) _data_->self);
	_data_->_tmp5_ = _data_->_tmp4_;
	_data_->_tmp6_ = rygel_media_export_media_cache_get_object_attribute_by_search_expression (_data_->_tmp2_, _data_->_tmp3_, _data_->_tmp5_, _data_->sort_criteria, (glong) _data_->offset, _data_->max_count, rygel_media_export_node_query_container_add_all_container (_data_->self), &_data_->_inner_error0_);
	_data_->data = _data_->_tmp6_;
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
		_g_object_unref0 (_data_->factory);
		_g_object_unref0 (_data_->children);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	{
		_data_->_meta_data_list = _data_->data;
		_data_->_tmp7_ = _data_->_meta_data_list;
		_data_->_tmp8_ = gee_collection_get_size ((GeeCollection*) _data_->_tmp7_);
		_data_->_tmp9_ = _data_->_tmp8_;
		_data_->_meta_data_size = _data_->_tmp9_;
		_data_->_meta_data_index = -1;
		while (TRUE) {
			_data_->_meta_data_index = _data_->_meta_data_index + 1;
			_data_->_tmp10_ = _data_->_meta_data_index;
			_data_->_tmp11_ = _data_->_meta_data_size;
			if (!(_data_->_tmp10_ < _data_->_tmp11_)) {
				break;
			}
			_data_->_tmp12_ = _data_->_meta_data_list;
			_data_->_tmp13_ = gee_list_get (_data_->_tmp12_, _data_->_meta_data_index);
			_data_->meta_data = (gchar*) _data_->_tmp13_;
			_data_->_tmp14_ = _data_->meta_data;
			if (g_strcmp0 (_data_->_tmp14_, "all_place_holder") == 0) {
				_data_->_tmp15_ = _data_->self->priv->_template;
				_data_->_tmp16_ = string_replace (_data_->_tmp15_, ",upnp:album,%s", "");
				_g_free0 (_data_->id);
				_data_->id = _data_->_tmp16_;
				_data_->_tmp17_ = _data_->id;
				_data_->_tmp18_ = rygel_media_export_query_container_factory_create_from_description_id (_data_->factory, _data_->_tmp17_, _ ("All"));
				_g_object_unref0 (_data_->container);
				_data_->container = (RygelMediaContainer*) _data_->_tmp18_;
			} else {
				_data_->_tmp19_ = _data_->meta_data;
				_data_->_tmp20_ = g_uri_escape_string (_data_->_tmp19_, "", TRUE);
				_g_free0 (_data_->id);
				_data_->id = _data_->_tmp20_;
				_data_->_tmp21_ = _data_->self->priv->_template;
				_data_->_tmp22_ = _data_->id;
				_data_->_tmp23_ = string_replace (_data_->_tmp21_, "%s", _data_->_tmp22_);
				_g_free0 (_data_->id);
				_data_->id = _data_->_tmp23_;
				_data_->_tmp24_ = _data_->id;
				_data_->_tmp25_ = _data_->meta_data;
				_data_->_tmp26_ = rygel_media_export_query_container_factory_create_from_description_id (_data_->factory, _data_->_tmp24_, _data_->_tmp25_);
				_g_object_unref0 (_data_->container);
				_data_->container = (RygelMediaContainer*) _data_->_tmp26_;
			}
			_data_->_tmp27_ = _data_->container;
			rygel_media_object_set_parent ((RygelMediaObject*) _data_->_tmp27_, (RygelMediaContainer*) _data_->self);
			_data_->_tmp28_ = _data_->container;
			gee_abstract_collection_add ((GeeAbstractCollection*) _data_->children, (RygelMediaObject*) _data_->_tmp28_);
			_g_object_unref0 (_data_->container);
			_g_free0 (_data_->id);
			_g_free0 (_data_->meta_data);
		}
	}
	_data_->result = _data_->children;
	_g_object_unref0 (_data_->data);
	_g_object_unref0 (_data_->factory);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static gint
rygel_media_export_node_query_container_real_count_children (RygelMediaExportDBContainer* base)
{
	RygelMediaExportNodeQueryContainer * self;
	gint _tmp9_ = -1;
	GError* _inner_error0_ = NULL;
	gint result;
	self = (RygelMediaExportNodeQueryContainer*) base;
	{
		GeeList* data = NULL;
		RygelMediaExportMediaCache* _tmp0_;
		const gchar* _tmp1_;
		RygelSearchExpression* _tmp2_;
		RygelSearchExpression* _tmp3_;
		GeeList* _tmp4_;
		gint _tmp5_;
		gint _tmp6_;
		_tmp0_ = ((RygelMediaExportDBContainer*) self)->media_db;
		_tmp1_ = self->priv->_attribute;
		_tmp2_ = rygel_media_export_query_container_get_expression ((RygelMediaExportQueryContainer*) self);
		_tmp3_ = _tmp2_;
		_tmp4_ = rygel_media_export_media_cache_get_object_attribute_by_search_expression (_tmp0_, _tmp1_, _tmp3_, "+dc:title", (glong) 0, (guint) -1, rygel_media_export_node_query_container_add_all_container (self), &_inner_error0_);
		data = _tmp4_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		_tmp5_ = gee_collection_get_size ((GeeCollection*) data);
		_tmp6_ = _tmp5_;
		result = _tmp6_;
		_g_object_unref0 (data);
		return result;
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp7_;
		const gchar* _tmp8_;
		_error_ = _inner_error0_;
		_inner_error0_ = NULL;
		_tmp7_ = _error_;
		_tmp8_ = _tmp7_->message;
		g_warning (_ ("Failed to get child count: %s"), _tmp8_);
		result = 0;
		_g_error_free0 (_error_);
		return result;
	}
	__finally0:
	g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
	g_clear_error (&_inner_error0_);
	return _tmp9_;
}

static gboolean
rygel_media_export_node_query_container_add_all_container (RygelMediaExportNodeQueryContainer* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = FALSE;
	return result;
}

static const gchar*
rygel_media_export_node_query_container_get_template (RygelMediaExportNodeQueryContainer* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_template;
	result = _tmp0_;
	return result;
}

static void
rygel_media_export_node_query_container_set_template (RygelMediaExportNodeQueryContainer* self,
                                                      const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = rygel_media_export_node_query_container_get_template (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_template);
		self->priv->_template = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, rygel_media_export_node_query_container_properties[RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_TEMPLATE_PROPERTY]);
	}
}

static const gchar*
rygel_media_export_node_query_container_get_attribute (RygelMediaExportNodeQueryContainer* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_attribute;
	result = _tmp0_;
	return result;
}

static void
rygel_media_export_node_query_container_set_attribute (RygelMediaExportNodeQueryContainer* self,
                                                       const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = rygel_media_export_node_query_container_get_attribute (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_attribute);
		self->priv->_attribute = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, rygel_media_export_node_query_container_properties[RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_ATTRIBUTE_PROPERTY]);
	}
}

static void
rygel_media_export_node_query_container_class_init (RygelMediaExportNodeQueryContainerClass * klass,
                                                    gpointer klass_data)
{
	rygel_media_export_node_query_container_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &RygelMediaExportNodeQueryContainer_private_offset);
	((RygelMediaContainerClass *) klass)->get_children = (void (*) (RygelMediaContainer*, guint, guint, const gchar*, GCancellable*, GAsyncReadyCallback, gpointer)) rygel_media_export_node_query_container_real_get_children;
	((RygelMediaContainerClass *) klass)->get_children_finish = (RygelMediaObjects* (*) (RygelMediaContainer*, GAsyncResult*, GError**)) rygel_media_export_node_query_container_get_children_finish;
	((RygelMediaExportDBContainerClass *) klass)->count_children = (gint (*) (RygelMediaExportDBContainer*)) rygel_media_export_node_query_container_real_count_children;
	G_OBJECT_CLASS (klass)->get_property = _vala_rygel_media_export_node_query_container_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_rygel_media_export_node_query_container_set_property;
	G_OBJECT_CLASS (klass)->finalize = rygel_media_export_node_query_container_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_TEMPLATE_PROPERTY, rygel_media_export_node_query_container_properties[RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_TEMPLATE_PROPERTY] = g_param_spec_string ("template", "template", "template", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_ATTRIBUTE_PROPERTY, rygel_media_export_node_query_container_properties[RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_ATTRIBUTE_PROPERTY] = g_param_spec_string ("attribute", "attribute", "attribute", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}

static void
rygel_media_export_node_query_container_instance_init (RygelMediaExportNodeQueryContainer * self,
                                                       gpointer klass)
{
	self->priv = rygel_media_export_node_query_container_get_instance_private (self);
}

static void
rygel_media_export_node_query_container_finalize (GObject * obj)
{
	RygelMediaExportNodeQueryContainer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER, RygelMediaExportNodeQueryContainer);
	_g_free0 (self->priv->_template);
	_g_free0 (self->priv->_attribute);
	G_OBJECT_CLASS (rygel_media_export_node_query_container_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
rygel_media_export_node_query_container_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelMediaExportNodeQueryContainerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_media_export_node_query_container_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RygelMediaExportNodeQueryContainer), 0, (GInstanceInitFunc) rygel_media_export_node_query_container_instance_init, NULL };
	GType rygel_media_export_node_query_container_type_id;
	rygel_media_export_node_query_container_type_id = g_type_register_static (RYGEL_MEDIA_EXPORT_TYPE_QUERY_CONTAINER, "RygelMediaExportNodeQueryContainer", &g_define_type_info, 0);
	RygelMediaExportNodeQueryContainer_private_offset = g_type_add_instance_private (rygel_media_export_node_query_container_type_id, sizeof (RygelMediaExportNodeQueryContainerPrivate));
	return rygel_media_export_node_query_container_type_id;
}

GType
rygel_media_export_node_query_container_get_type (void)
{
	static volatile gsize rygel_media_export_node_query_container_type_id__once = 0;
	if (g_once_init_enter (&rygel_media_export_node_query_container_type_id__once)) {
		GType rygel_media_export_node_query_container_type_id;
		rygel_media_export_node_query_container_type_id = rygel_media_export_node_query_container_get_type_once ();
		g_once_init_leave (&rygel_media_export_node_query_container_type_id__once, rygel_media_export_node_query_container_type_id);
	}
	return rygel_media_export_node_query_container_type_id__once;
}

static void
_vala_rygel_media_export_node_query_container_get_property (GObject * object,
                                                            guint property_id,
                                                            GValue * value,
                                                            GParamSpec * pspec)
{
	RygelMediaExportNodeQueryContainer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER, RygelMediaExportNodeQueryContainer);
	switch (property_id) {
		case RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_TEMPLATE_PROPERTY:
		g_value_set_string (value, rygel_media_export_node_query_container_get_template (self));
		break;
		case RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_ATTRIBUTE_PROPERTY:
		g_value_set_string (value, rygel_media_export_node_query_container_get_attribute (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_rygel_media_export_node_query_container_set_property (GObject * object,
                                                            guint property_id,
                                                            const GValue * value,
                                                            GParamSpec * pspec)
{
	RygelMediaExportNodeQueryContainer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, RYGEL_MEDIA_EXPORT_TYPE_NODE_QUERY_CONTAINER, RygelMediaExportNodeQueryContainer);
	switch (property_id) {
		case RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_TEMPLATE_PROPERTY:
		rygel_media_export_node_query_container_set_template (self, g_value_get_string (value));
		break;
		case RYGEL_MEDIA_EXPORT_NODE_QUERY_CONTAINER_ATTRIBUTE_PROPERTY:
		rygel_media_export_node_query_container_set_attribute (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

