/* rygel-last-change-st-done.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-last-change-st-done.vala, do not modify */

/*
 * Copyright (C) 2012 Openismus GmbH
 *
 * Author: Krzesimir Nowak <krnowak@openismus.com>
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "rygel-server.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define RYGEL_TYPE_LAST_CHANGE_ENTRY (rygel_last_change_entry_get_type ())
#define RYGEL_LAST_CHANGE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_LAST_CHANGE_ENTRY, RygelLastChangeEntry))
#define RYGEL_LAST_CHANGE_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_LAST_CHANGE_ENTRY, RygelLastChangeEntryClass))
#define RYGEL_IS_LAST_CHANGE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_LAST_CHANGE_ENTRY))
#define RYGEL_IS_LAST_CHANGE_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_LAST_CHANGE_ENTRY))
#define RYGEL_LAST_CHANGE_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_LAST_CHANGE_ENTRY, RygelLastChangeEntryClass))

typedef struct _RygelLastChangeEntry RygelLastChangeEntry;
typedef struct _RygelLastChangeEntryClass RygelLastChangeEntryClass;
typedef struct _RygelLastChangeEntryPrivate RygelLastChangeEntryPrivate;

#define RYGEL_TYPE_LAST_CHANGE_ST_DONE (rygel_last_change_st_done_get_type ())
#define RYGEL_LAST_CHANGE_ST_DONE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_LAST_CHANGE_ST_DONE, RygelLastChangeStDone))
#define RYGEL_LAST_CHANGE_ST_DONE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_LAST_CHANGE_ST_DONE, RygelLastChangeStDoneClass))
#define RYGEL_IS_LAST_CHANGE_ST_DONE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_LAST_CHANGE_ST_DONE))
#define RYGEL_IS_LAST_CHANGE_ST_DONE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_LAST_CHANGE_ST_DONE))
#define RYGEL_LAST_CHANGE_ST_DONE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_LAST_CHANGE_ST_DONE, RygelLastChangeStDoneClass))

typedef struct _RygelLastChangeStDone RygelLastChangeStDone;
typedef struct _RygelLastChangeStDoneClass RygelLastChangeStDoneClass;
typedef struct _RygelLastChangeStDonePrivate RygelLastChangeStDonePrivate;
enum  {
	RYGEL_LAST_CHANGE_ST_DONE_0_PROPERTY,
	RYGEL_LAST_CHANGE_ST_DONE_NUM_PROPERTIES
};
static GParamSpec* rygel_last_change_st_done_properties[RYGEL_LAST_CHANGE_ST_DONE_NUM_PROPERTIES];

struct _RygelLastChangeEntry {
	GObject parent_instance;
	RygelLastChangeEntryPrivate * priv;
};

struct _RygelLastChangeEntryClass {
	GObjectClass parent_class;
	gchar* (*additional_info) (RygelLastChangeEntry* self);
};

struct _RygelLastChangeStDone {
	RygelLastChangeEntry parent_instance;
	RygelLastChangeStDonePrivate * priv;
};

struct _RygelLastChangeStDoneClass {
	RygelLastChangeEntryClass parent_class;
};

static gpointer rygel_last_change_st_done_parent_class = NULL;

VALA_EXTERN GType rygel_last_change_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType rygel_last_change_st_done_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gchar* rygel_last_change_entry_additional_info (RygelLastChangeEntry* self);
VALA_EXTERN RygelLastChangeStDone* rygel_last_change_st_done_new (const gchar* id,
                                                      guint update_id);
VALA_EXTERN RygelLastChangeStDone* rygel_last_change_st_done_construct (GType object_type,
                                                            const gchar* id,
                                                            guint update_id);
VALA_EXTERN RygelLastChangeEntry* rygel_last_change_entry_construct (GType object_type,
                                                         const gchar* tag,
                                                         const gchar* id,
                                                         guint update_id);
static gchar* rygel_last_change_st_done_real_additional_info (RygelLastChangeEntry* base);
static GType rygel_last_change_st_done_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

RygelLastChangeStDone*
rygel_last_change_st_done_construct (GType object_type,
                                     const gchar* id,
                                     guint update_id)
{
	RygelLastChangeStDone * self = NULL;
	g_return_val_if_fail (id != NULL, NULL);
	self = (RygelLastChangeStDone*) rygel_last_change_entry_construct (object_type, "stDone", id, update_id);
	return self;
}

RygelLastChangeStDone*
rygel_last_change_st_done_new (const gchar* id,
                               guint update_id)
{
	return rygel_last_change_st_done_construct (RYGEL_TYPE_LAST_CHANGE_ST_DONE, id, update_id);
}

static gchar*
rygel_last_change_st_done_real_additional_info (RygelLastChangeEntry* base)
{
	RygelLastChangeStDone * self;
	gchar* _tmp0_;
	gchar* result;
	self = (RygelLastChangeStDone*) base;
	_tmp0_ = g_strdup ("");
	result = _tmp0_;
	return result;
}

static void
rygel_last_change_st_done_class_init (RygelLastChangeStDoneClass * klass,
                                      gpointer klass_data)
{
	rygel_last_change_st_done_parent_class = g_type_class_peek_parent (klass);
	((RygelLastChangeEntryClass *) klass)->additional_info = (gchar* (*) (RygelLastChangeEntry*)) rygel_last_change_st_done_real_additional_info;
}

static void
rygel_last_change_st_done_instance_init (RygelLastChangeStDone * self,
                                         gpointer klass)
{
}

 G_GNUC_NO_INLINE static GType
rygel_last_change_st_done_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelLastChangeStDoneClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_last_change_st_done_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RygelLastChangeStDone), 0, (GInstanceInitFunc) rygel_last_change_st_done_instance_init, NULL };
	GType rygel_last_change_st_done_type_id;
	rygel_last_change_st_done_type_id = g_type_register_static (RYGEL_TYPE_LAST_CHANGE_ENTRY, "RygelLastChangeStDone", &g_define_type_info, 0);
	return rygel_last_change_st_done_type_id;
}

GType
rygel_last_change_st_done_get_type (void)
{
	static volatile gsize rygel_last_change_st_done_type_id__once = 0;
	if (g_once_init_enter (&rygel_last_change_st_done_type_id__once)) {
		GType rygel_last_change_st_done_type_id;
		rygel_last_change_st_done_type_id = rygel_last_change_st_done_get_type_once ();
		g_once_init_leave (&rygel_last_change_st_done_type_id__once, rygel_last_change_st_done_type_id);
	}
	return rygel_last_change_st_done_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

