/* rygel-free-desktop-interfaces.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-free-desktop-interfaces.vala, do not modify */

/*
 * Copyright (C) 2009,2010 Nokia Corporation.
 *
 * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
 *                               <zeeshan.ali@nokia.com>
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include "rygel-external.h"

#define RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_SERVICE "org.freedesktop.DBus"
#define RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_PATH "/org/freedesktop/DBus"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_DBUS_OBJECT (rygel_external_free_desktop_dbus_object_get_type ())
#define RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_DBUS_OBJECT, RygelExternalFreeDesktopDBusObject))
#define RYGEL_EXTERNAL_FREE_DESKTOP_IS_DBUS_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_DBUS_OBJECT))
#define RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_DBUS_OBJECT, RygelExternalFreeDesktopDBusObjectIface))

typedef struct _RygelExternalFreeDesktopDBusObject RygelExternalFreeDesktopDBusObject;
typedef struct _RygelExternalFreeDesktopDBusObjectIface RygelExternalFreeDesktopDBusObjectIface;

#define RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_DBUS_OBJECT_PROXY (rygel_external_free_desktop_dbus_object_proxy_get_type ())
enum  {
	RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_NAME_OWNER_CHANGED_SIGNAL,
	RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_NUM_SIGNALS
};
static guint rygel_external_free_desktop_dbus_object_signals[RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_NUM_SIGNALS] = {0};
typedef GDBusProxy RygelExternalFreeDesktopDBusObjectProxy;
typedef GDBusProxyClass RygelExternalFreeDesktopDBusObjectProxyClass;
#define _g_free0(var) (var = (g_free (var), NULL))

#define RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_PROPERTIES (rygel_external_free_desktop_properties_get_type ())
#define RYGEL_EXTERNAL_FREE_DESKTOP_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_PROPERTIES, RygelExternalFreeDesktopProperties))
#define RYGEL_EXTERNAL_FREE_DESKTOP_IS_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_PROPERTIES))
#define RYGEL_EXTERNAL_FREE_DESKTOP_PROPERTIES_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_PROPERTIES, RygelExternalFreeDesktopPropertiesIface))

typedef struct _RygelExternalFreeDesktopProperties RygelExternalFreeDesktopProperties;
typedef struct _RygelExternalFreeDesktopPropertiesIface RygelExternalFreeDesktopPropertiesIface;

#define RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_PROPERTIES_PROXY (rygel_external_free_desktop_properties_proxy_get_type ())
typedef GDBusProxy RygelExternalFreeDesktopPropertiesProxy;
typedef GDBusProxyClass RygelExternalFreeDesktopPropertiesProxyClass;

struct _RygelExternalFreeDesktopDBusObjectIface {
	GTypeInterface parent_iface;
	void (*list_names) (RygelExternalFreeDesktopDBusObject* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar** (*list_names_finish) (RygelExternalFreeDesktopDBusObject* self, GAsyncResult* _res_, gint* result_length1, GError** error);
	void (*list_activatable_names) (RygelExternalFreeDesktopDBusObject* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar** (*list_activatable_names_finish) (RygelExternalFreeDesktopDBusObject* self, GAsyncResult* _res_, gint* result_length1, GError** error);
};

struct _RygelExternalFreeDesktopPropertiesIface {
	GTypeInterface parent_iface;
	void (*get_all) (RygelExternalFreeDesktopProperties* self, const gchar* iface, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GHashTable* (*get_all_finish) (RygelExternalFreeDesktopProperties* self, GAsyncResult* _res_, GError** error);
};

VALA_EXTERN GType rygel_external_free_desktop_dbus_object_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint rygel_external_free_desktop_dbus_object_register_object (void* object,
                                                               GDBusConnection* connection,
                                                               const gchar* path,
                                                               GError** error);
VALA_EXTERN GType rygel_external_free_desktop_dbus_object_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void rygel_external_free_desktop_dbus_object_list_names (RygelExternalFreeDesktopDBusObject* self,
                                                         GAsyncReadyCallback _callback_,
                                                         gpointer _user_data_);
VALA_EXTERN gchar** rygel_external_free_desktop_dbus_object_list_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                                   GAsyncResult* _res_,
                                                                   gint* result_length1,
                                                                   GError** error);
VALA_EXTERN void rygel_external_free_desktop_dbus_object_list_activatable_names (RygelExternalFreeDesktopDBusObject* self,
                                                                     GAsyncReadyCallback _callback_,
                                                                     gpointer _user_data_);
VALA_EXTERN gchar** rygel_external_free_desktop_dbus_object_list_activatable_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                                               GAsyncResult* _res_,
                                                                               gint* result_length1,
                                                                               GError** error);
static void g_cclosure_user_marshal_VOID__STRING_STRING_STRING (GClosure * closure,
                                                         GValue * return_value,
                                                         guint n_param_values,
                                                         const GValue * param_values,
                                                         gpointer invocation_hint,
                                                         gpointer marshal_data);
static GType rygel_external_free_desktop_dbus_object_get_type_once (void);
static void rygel_external_free_desktop_dbus_object_proxy_g_signal (GDBusProxy* proxy,
                                                             const gchar* sender_name,
                                                             const gchar* signal_name,
                                                             GVariant* parameters);
static void _dbus_handle_rygel_external_free_desktop_dbus_object_name_owner_changed (RygelExternalFreeDesktopDBusObject* self,
                                                                              GVariant* parameters);
static void _vala_g_async_ready_callback (GObject *source_object,
                                   GAsyncResult *res,
                                   void *user_data);
static void rygel_external_free_desktop_dbus_object_proxy_list_names_async (RygelExternalFreeDesktopDBusObject* self,
                                                                     GAsyncReadyCallback _callback_,
                                                                     gpointer _user_data_);
static gchar** rygel_external_free_desktop_dbus_object_proxy_list_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                                         GAsyncResult* _res_,
                                                                         gint* result_length1,
                                                                         GError** error);
static void rygel_external_free_desktop_dbus_object_proxy_list_activatable_names_async (RygelExternalFreeDesktopDBusObject* self,
                                                                                 GAsyncReadyCallback _callback_,
                                                                                 gpointer _user_data_);
static gchar** rygel_external_free_desktop_dbus_object_proxy_list_activatable_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                                                     GAsyncResult* _res_,
                                                                                     gint* result_length1,
                                                                                     GError** error);
static void rygel_external_free_desktop_dbus_object_proxy_rygel_external_free_desktop_dbus_object_interface_init (RygelExternalFreeDesktopDBusObjectIface* iface);
static void rygel_external_free_desktop_dbus_object_dbus_interface_method_call (GDBusConnection* connection,
                                                                         const gchar* sender,
                                                                         const gchar* object_path,
                                                                         const gchar* interface_name,
                                                                         const gchar* method_name,
                                                                         GVariant* parameters,
                                                                         GDBusMethodInvocation* invocation,
                                                                         gpointer user_data);
static GVariant* rygel_external_free_desktop_dbus_object_dbus_interface_get_property (GDBusConnection* connection,
                                                                               const gchar* sender,
                                                                               const gchar* object_path,
                                                                               const gchar* interface_name,
                                                                               const gchar* property_name,
                                                                               GError** error,
                                                                               gpointer user_data);
static gboolean rygel_external_free_desktop_dbus_object_dbus_interface_set_property (GDBusConnection* connection,
                                                                              const gchar* sender,
                                                                              const gchar* object_path,
                                                                              const gchar* interface_name,
                                                                              const gchar* property_name,
                                                                              GVariant* value,
                                                                              GError** error,
                                                                              gpointer user_data);
static void _dbus_rygel_external_free_desktop_dbus_object_name_owner_changed (GObject* _sender,
                                                                       const gchar* name,
                                                                       const gchar* old_owner,
                                                                       const gchar* new_owner,
                                                                       gpointer* _data);
static void _rygel_external_free_desktop_dbus_object_unregister_object (gpointer user_data);
VALA_EXTERN GType rygel_external_free_desktop_properties_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint rygel_external_free_desktop_properties_register_object (void* object,
                                                              GDBusConnection* connection,
                                                              const gchar* path,
                                                              GError** error);
VALA_EXTERN GType rygel_external_free_desktop_properties_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void rygel_external_free_desktop_properties_get_all (RygelExternalFreeDesktopProperties* self,
                                                     const gchar* iface,
                                                     GAsyncReadyCallback _callback_,
                                                     gpointer _user_data_);
VALA_EXTERN GHashTable* rygel_external_free_desktop_properties_get_all_finish (RygelExternalFreeDesktopProperties* self,
                                                                   GAsyncResult* _res_,
                                                                   GError** error);
static GType rygel_external_free_desktop_properties_get_type_once (void);
static void rygel_external_free_desktop_properties_proxy_g_signal (GDBusProxy* proxy,
                                                            const gchar* sender_name,
                                                            const gchar* signal_name,
                                                            GVariant* parameters);
static void rygel_external_free_desktop_properties_proxy_get_all_async (RygelExternalFreeDesktopProperties* self,
                                                                 const gchar* iface,
                                                                 GAsyncReadyCallback _callback_,
                                                                 gpointer _user_data_);
static GHashTable* rygel_external_free_desktop_properties_proxy_get_all_finish (RygelExternalFreeDesktopProperties* self,
                                                                         GAsyncResult* _res_,
                                                                         GError** error);
static void rygel_external_free_desktop_properties_proxy_rygel_external_free_desktop_properties_interface_init (RygelExternalFreeDesktopPropertiesIface* iface);
static void rygel_external_free_desktop_properties_dbus_interface_method_call (GDBusConnection* connection,
                                                                        const gchar* sender,
                                                                        const gchar* object_path,
                                                                        const gchar* interface_name,
                                                                        const gchar* method_name,
                                                                        GVariant* parameters,
                                                                        GDBusMethodInvocation* invocation,
                                                                        gpointer user_data);
static GVariant* rygel_external_free_desktop_properties_dbus_interface_get_property (GDBusConnection* connection,
                                                                              const gchar* sender,
                                                                              const gchar* object_path,
                                                                              const gchar* interface_name,
                                                                              const gchar* property_name,
                                                                              GError** error,
                                                                              gpointer user_data);
static gboolean rygel_external_free_desktop_properties_dbus_interface_set_property (GDBusConnection* connection,
                                                                             const gchar* sender,
                                                                             const gchar* object_path,
                                                                             const gchar* interface_name,
                                                                             const gchar* property_name,
                                                                             GVariant* value,
                                                                             GError** error,
                                                                             gpointer user_data);
static void _rygel_external_free_desktop_properties_unregister_object (gpointer user_data);

static const GDBusMethodInfo * const _rygel_external_free_desktop_dbus_object_dbus_method_info[] = {NULL};
static const GDBusArgInfo _rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed_name = {-1, "name", "s", NULL};
static const GDBusArgInfo _rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed_old_owner = {-1, "old_owner", "s", NULL};
static const GDBusArgInfo _rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed_new_owner = {-1, "new_owner", "s", NULL};
static const GDBusArgInfo * const _rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed[] = {&_rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed_name, &_rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed_old_owner, &_rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed_new_owner, NULL};
static const GDBusSignalInfo _rygel_external_free_desktop_dbus_object_dbus_signal_info_name_owner_changed = {-1, "NameOwnerChanged", (GDBusArgInfo **) (&_rygel_external_free_desktop_dbus_object_dbus_arg_info_name_owner_changed), NULL};
static const GDBusSignalInfo * const _rygel_external_free_desktop_dbus_object_dbus_signal_info[] = {&_rygel_external_free_desktop_dbus_object_dbus_signal_info_name_owner_changed, NULL};
static const GDBusPropertyInfo * const _rygel_external_free_desktop_dbus_object_dbus_property_info[] = {NULL};
static const GDBusInterfaceInfo _rygel_external_free_desktop_dbus_object_dbus_interface_info = {-1, "org.freedesktop.DBus", (GDBusMethodInfo **) (&_rygel_external_free_desktop_dbus_object_dbus_method_info), (GDBusSignalInfo **) (&_rygel_external_free_desktop_dbus_object_dbus_signal_info), (GDBusPropertyInfo **) (&_rygel_external_free_desktop_dbus_object_dbus_property_info), NULL};
static const GDBusInterfaceVTable _rygel_external_free_desktop_dbus_object_dbus_interface_vtable = {rygel_external_free_desktop_dbus_object_dbus_interface_method_call, rygel_external_free_desktop_dbus_object_dbus_interface_get_property, rygel_external_free_desktop_dbus_object_dbus_interface_set_property};
static const GDBusMethodInfo * const _rygel_external_free_desktop_properties_dbus_method_info[] = {NULL};
static const GDBusSignalInfo * const _rygel_external_free_desktop_properties_dbus_signal_info[] = {NULL};
static const GDBusPropertyInfo * const _rygel_external_free_desktop_properties_dbus_property_info[] = {NULL};
static const GDBusInterfaceInfo _rygel_external_free_desktop_properties_dbus_interface_info = {-1, "org.freedesktop.DBus.Properties", (GDBusMethodInfo **) (&_rygel_external_free_desktop_properties_dbus_method_info), (GDBusSignalInfo **) (&_rygel_external_free_desktop_properties_dbus_signal_info), (GDBusPropertyInfo **) (&_rygel_external_free_desktop_properties_dbus_property_info), NULL};
static const GDBusInterfaceVTable _rygel_external_free_desktop_properties_dbus_interface_vtable = {rygel_external_free_desktop_properties_dbus_interface_method_call, rygel_external_free_desktop_properties_dbus_interface_get_property, rygel_external_free_desktop_properties_dbus_interface_set_property};

void
rygel_external_free_desktop_dbus_object_list_names (RygelExternalFreeDesktopDBusObject* self,
                                                    GAsyncReadyCallback _callback_,
                                                    gpointer _user_data_)
{
	RygelExternalFreeDesktopDBusObjectIface* _iface_;
	_iface_ = RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE (self);
	if (_iface_->list_names) {
		_iface_->list_names (self, _callback_, _user_data_);
	}
}

gchar**
rygel_external_free_desktop_dbus_object_list_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                           GAsyncResult* _res_,
                                                           gint* result_length1,
                                                           GError** error)
{
	RygelExternalFreeDesktopDBusObjectIface* _iface_;
	_iface_ = RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE (self);
	if (_iface_->list_names_finish) {
		return _iface_->list_names_finish (self, _res_, result_length1, error);
	}
	return NULL;
}

void
rygel_external_free_desktop_dbus_object_list_activatable_names (RygelExternalFreeDesktopDBusObject* self,
                                                                GAsyncReadyCallback _callback_,
                                                                gpointer _user_data_)
{
	RygelExternalFreeDesktopDBusObjectIface* _iface_;
	_iface_ = RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE (self);
	if (_iface_->list_activatable_names) {
		_iface_->list_activatable_names (self, _callback_, _user_data_);
	}
}

gchar**
rygel_external_free_desktop_dbus_object_list_activatable_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                                       GAsyncResult* _res_,
                                                                       gint* result_length1,
                                                                       GError** error)
{
	RygelExternalFreeDesktopDBusObjectIface* _iface_;
	_iface_ = RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE (self);
	if (_iface_->list_activatable_names_finish) {
		return _iface_->list_activatable_names_finish (self, _res_, result_length1, error);
	}
	return NULL;
}

static void
g_cclosure_user_marshal_VOID__STRING_STRING_STRING (GClosure * closure,
                                                    GValue * return_value,
                                                    guint n_param_values,
                                                    const GValue * param_values,
                                                    gpointer invocation_hint,
                                                    gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING) (gpointer data1, const char* arg_1, const char* arg_2, const char* arg_3, gpointer data2);
	register GMarshalFunc_VOID__STRING_STRING_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 4);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__STRING_STRING_STRING) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_string (param_values + 1), g_value_get_string (param_values + 2), g_value_get_string (param_values + 3), data2);
}

static void
rygel_external_free_desktop_dbus_object_default_init (RygelExternalFreeDesktopDBusObjectIface * iface,
                                                      gpointer iface_data)
{
	rygel_external_free_desktop_dbus_object_signals[RYGEL_EXTERNAL_FREE_DESKTOP_DBUS_OBJECT_NAME_OWNER_CHANGED_SIGNAL] = g_signal_new ("name-owner-changed", RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_DBUS_OBJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__STRING_STRING_STRING, G_TYPE_NONE, 3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
}

 G_GNUC_NO_INLINE static GType
rygel_external_free_desktop_dbus_object_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelExternalFreeDesktopDBusObjectIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_external_free_desktop_dbus_object_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType rygel_external_free_desktop_dbus_object_type_id;
	rygel_external_free_desktop_dbus_object_type_id = g_type_register_static (G_TYPE_INTERFACE, "RygelExternalFreeDesktopDBusObject", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (rygel_external_free_desktop_dbus_object_type_id, G_TYPE_OBJECT);
	g_type_set_qdata (rygel_external_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-proxy-type"), (void*) rygel_external_free_desktop_dbus_object_proxy_get_type);
	g_type_set_qdata (rygel_external_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-interface-name"), "org.freedesktop.DBus");
	g_type_set_qdata (rygel_external_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-interface-info"), (void*) (&_rygel_external_free_desktop_dbus_object_dbus_interface_info));
	g_type_set_qdata (rygel_external_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-register-object"), (void*) rygel_external_free_desktop_dbus_object_register_object);
	return rygel_external_free_desktop_dbus_object_type_id;
}

GType
rygel_external_free_desktop_dbus_object_get_type (void)
{
	static volatile gsize rygel_external_free_desktop_dbus_object_type_id__once = 0;
	if (g_once_init_enter (&rygel_external_free_desktop_dbus_object_type_id__once)) {
		GType rygel_external_free_desktop_dbus_object_type_id;
		rygel_external_free_desktop_dbus_object_type_id = rygel_external_free_desktop_dbus_object_get_type_once ();
		g_once_init_leave (&rygel_external_free_desktop_dbus_object_type_id__once, rygel_external_free_desktop_dbus_object_type_id);
	}
	return rygel_external_free_desktop_dbus_object_type_id__once;
}

G_DEFINE_TYPE_EXTENDED (RygelExternalFreeDesktopDBusObjectProxy, rygel_external_free_desktop_dbus_object_proxy, G_TYPE_DBUS_PROXY, 0, G_IMPLEMENT_INTERFACE (RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_DBUS_OBJECT, rygel_external_free_desktop_dbus_object_proxy_rygel_external_free_desktop_dbus_object_interface_init) )
static void
rygel_external_free_desktop_dbus_object_proxy_class_init (RygelExternalFreeDesktopDBusObjectProxyClass* klass)
{
	G_DBUS_PROXY_CLASS (klass)->g_signal = rygel_external_free_desktop_dbus_object_proxy_g_signal;
}

static void
_dbus_handle_rygel_external_free_desktop_dbus_object_name_owner_changed (RygelExternalFreeDesktopDBusObject* self,
                                                                         GVariant* parameters)
{
	GVariantIter _arguments_iter;
	gchar* name = NULL;
	GVariant* _tmp121_;
	gchar* old_owner = NULL;
	GVariant* _tmp122_;
	gchar* new_owner = NULL;
	GVariant* _tmp123_;
	g_variant_iter_init (&_arguments_iter, parameters);
	_tmp121_ = g_variant_iter_next_value (&_arguments_iter);
	name = g_variant_dup_string (_tmp121_, NULL);
	g_variant_unref (_tmp121_);
	_tmp122_ = g_variant_iter_next_value (&_arguments_iter);
	old_owner = g_variant_dup_string (_tmp122_, NULL);
	g_variant_unref (_tmp122_);
	_tmp123_ = g_variant_iter_next_value (&_arguments_iter);
	new_owner = g_variant_dup_string (_tmp123_, NULL);
	g_variant_unref (_tmp123_);
	g_signal_emit_by_name (self, "name-owner-changed", name, old_owner, new_owner);
	_g_free0 (name);
	_g_free0 (old_owner);
	_g_free0 (new_owner);
}

static void
rygel_external_free_desktop_dbus_object_proxy_g_signal (GDBusProxy* proxy,
                                                        const gchar* sender_name,
                                                        const gchar* signal_name,
                                                        GVariant* parameters)
{
	if (strcmp (signal_name, "NameOwnerChanged") == 0) {
		_dbus_handle_rygel_external_free_desktop_dbus_object_name_owner_changed ((RygelExternalFreeDesktopDBusObject*) proxy, parameters);
	}
}

static void
rygel_external_free_desktop_dbus_object_proxy_init (RygelExternalFreeDesktopDBusObjectProxy* self)
{
	g_dbus_proxy_set_interface_info (G_DBUS_PROXY (self), (GDBusInterfaceInfo *) (&_rygel_external_free_desktop_dbus_object_dbus_interface_info));
}

static void
_vala_g_async_ready_callback (GObject *source_object,
                              GAsyncResult *res,
                              void *user_data)
{
	g_task_return_pointer (user_data, g_object_ref (res), g_object_unref);
	g_object_unref (user_data);
}

static void
rygel_external_free_desktop_dbus_object_proxy_list_names_async (RygelExternalFreeDesktopDBusObject* self,
                                                                GAsyncReadyCallback _callback_,
                                                                gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	G_DBUS_ERROR;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.DBus", "ListNames");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static gchar**
rygel_external_free_desktop_dbus_object_proxy_list_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                                 GAsyncResult* _res_,
                                                                 gint* result_length1,
                                                                 GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	GVariant *_reply;
	GVariantIter _reply_iter;
	gchar** _result = NULL;
	gint _result_length1;
	GVariant* _tmp124_;
	gchar** _tmp125_;
	gint _tmp125__length;
	gint _tmp125__size;
	gint _tmp125__length1;
	GVariantIter _tmp126_;
	GVariant* _tmp127_;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		return NULL;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		g_object_unref (_reply_message);
		return NULL;
	}
	_reply = g_dbus_message_get_body (_reply_message);
	g_variant_iter_init (&_reply_iter, _reply);
	_result_length1 = 0;
	_tmp124_ = g_variant_iter_next_value (&_reply_iter);
	_tmp125_ = g_new (gchar*, 5);
	_tmp125__length = 0;
	_tmp125__size = 4;
	_tmp125__length1 = 0;
	g_variant_iter_init (&_tmp126_, _tmp124_);
	for (; (_tmp127_ = g_variant_iter_next_value (&_tmp126_)) != NULL; _tmp125__length1++) {
		if (_tmp125__size == _tmp125__length) {
			_tmp125__size = 2 * _tmp125__size;
			_tmp125_ = g_renew (gchar*, _tmp125_, _tmp125__size + 1);
		}
		_tmp125_[_tmp125__length++] = g_variant_dup_string (_tmp127_, NULL);
		g_variant_unref (_tmp127_);
	}
	_result_length1 = _tmp125__length1;
	_tmp125_[_tmp125__length] = NULL;
	_result = _tmp125_;
	g_variant_unref (_tmp124_);
	*result_length1 = _result_length1;
	g_object_unref (_reply_message);
	return _result;
}

static void
rygel_external_free_desktop_dbus_object_proxy_list_activatable_names_async (RygelExternalFreeDesktopDBusObject* self,
                                                                            GAsyncReadyCallback _callback_,
                                                                            gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	G_DBUS_ERROR;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.DBus", "ListActivatableNames");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static gchar**
rygel_external_free_desktop_dbus_object_proxy_list_activatable_names_finish (RygelExternalFreeDesktopDBusObject* self,
                                                                             GAsyncResult* _res_,
                                                                             gint* result_length1,
                                                                             GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	GVariant *_reply;
	GVariantIter _reply_iter;
	gchar** _result = NULL;
	gint _result_length1;
	GVariant* _tmp128_;
	gchar** _tmp129_;
	gint _tmp129__length;
	gint _tmp129__size;
	gint _tmp129__length1;
	GVariantIter _tmp130_;
	GVariant* _tmp131_;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		return NULL;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		g_object_unref (_reply_message);
		return NULL;
	}
	_reply = g_dbus_message_get_body (_reply_message);
	g_variant_iter_init (&_reply_iter, _reply);
	_result_length1 = 0;
	_tmp128_ = g_variant_iter_next_value (&_reply_iter);
	_tmp129_ = g_new (gchar*, 5);
	_tmp129__length = 0;
	_tmp129__size = 4;
	_tmp129__length1 = 0;
	g_variant_iter_init (&_tmp130_, _tmp128_);
	for (; (_tmp131_ = g_variant_iter_next_value (&_tmp130_)) != NULL; _tmp129__length1++) {
		if (_tmp129__size == _tmp129__length) {
			_tmp129__size = 2 * _tmp129__size;
			_tmp129_ = g_renew (gchar*, _tmp129_, _tmp129__size + 1);
		}
		_tmp129_[_tmp129__length++] = g_variant_dup_string (_tmp131_, NULL);
		g_variant_unref (_tmp131_);
	}
	_result_length1 = _tmp129__length1;
	_tmp129_[_tmp129__length] = NULL;
	_result = _tmp129_;
	g_variant_unref (_tmp128_);
	*result_length1 = _result_length1;
	g_object_unref (_reply_message);
	return _result;
}

static void
rygel_external_free_desktop_dbus_object_proxy_rygel_external_free_desktop_dbus_object_interface_init (RygelExternalFreeDesktopDBusObjectIface* iface)
{
	iface->list_names = rygel_external_free_desktop_dbus_object_proxy_list_names_async;
	iface->list_names_finish = rygel_external_free_desktop_dbus_object_proxy_list_names_finish;
	iface->list_activatable_names = rygel_external_free_desktop_dbus_object_proxy_list_activatable_names_async;
	iface->list_activatable_names_finish = rygel_external_free_desktop_dbus_object_proxy_list_activatable_names_finish;
}

static void
rygel_external_free_desktop_dbus_object_dbus_interface_method_call (GDBusConnection* connection,
                                                                    const gchar* sender,
                                                                    const gchar* object_path,
                                                                    const gchar* interface_name,
                                                                    const gchar* method_name,
                                                                    GVariant* parameters,
                                                                    GDBusMethodInvocation* invocation,
                                                                    gpointer user_data)
{
	g_object_unref (invocation);
}

static GVariant*
rygel_external_free_desktop_dbus_object_dbus_interface_get_property (GDBusConnection* connection,
                                                                     const gchar* sender,
                                                                     const gchar* object_path,
                                                                     const gchar* interface_name,
                                                                     const gchar* property_name,
                                                                     GError** error,
                                                                     gpointer user_data)
{
	return NULL;
}

static gboolean
rygel_external_free_desktop_dbus_object_dbus_interface_set_property (GDBusConnection* connection,
                                                                     const gchar* sender,
                                                                     const gchar* object_path,
                                                                     const gchar* interface_name,
                                                                     const gchar* property_name,
                                                                     GVariant* value,
                                                                     GError** error,
                                                                     gpointer user_data)
{
	return FALSE;
}

static void
_dbus_rygel_external_free_desktop_dbus_object_name_owner_changed (GObject* _sender,
                                                                  const gchar* name,
                                                                  const gchar* old_owner,
                                                                  const gchar* new_owner,
                                                                  gpointer* _data)
{
	GDBusConnection * _connection;
	const gchar * _path;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	_connection = _data[1];
	_path = _data[2];
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (name));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (old_owner));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (new_owner));
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_connection_emit_signal (_connection, NULL, _path, "org.freedesktop.DBus", "NameOwnerChanged", _arguments, NULL);
}

guint
rygel_external_free_desktop_dbus_object_register_object (gpointer object,
                                                         GDBusConnection* connection,
                                                         const gchar* path,
                                                         GError** error)
{
	guint result;
	gpointer *data;
	data = g_new (gpointer, 3);
	data[0] = g_object_ref (object);
	data[1] = g_object_ref (connection);
	data[2] = g_strdup (path);
	result = g_dbus_connection_register_object (connection, path, (GDBusInterfaceInfo *) (&_rygel_external_free_desktop_dbus_object_dbus_interface_info), &_rygel_external_free_desktop_dbus_object_dbus_interface_vtable, data, _rygel_external_free_desktop_dbus_object_unregister_object, error);
	if (!result) {
		return 0;
	}
	g_signal_connect (object, "name-owner-changed", (GCallback) _dbus_rygel_external_free_desktop_dbus_object_name_owner_changed, data);
	return result;
}

static void
_rygel_external_free_desktop_dbus_object_unregister_object (gpointer user_data)
{
	gpointer* data;
	data = user_data;
	g_signal_handlers_disconnect_by_func (data[0], _dbus_rygel_external_free_desktop_dbus_object_name_owner_changed, data);
	g_object_unref (data[0]);
	g_object_unref (data[1]);
	g_free (data[2]);
	g_free (data);
}

void
rygel_external_free_desktop_properties_get_all (RygelExternalFreeDesktopProperties* self,
                                                const gchar* iface,
                                                GAsyncReadyCallback _callback_,
                                                gpointer _user_data_)
{
	RygelExternalFreeDesktopPropertiesIface* _iface_;
	_iface_ = RYGEL_EXTERNAL_FREE_DESKTOP_PROPERTIES_GET_INTERFACE (self);
	if (_iface_->get_all) {
		_iface_->get_all (self, iface, _callback_, _user_data_);
	}
}

GHashTable*
rygel_external_free_desktop_properties_get_all_finish (RygelExternalFreeDesktopProperties* self,
                                                       GAsyncResult* _res_,
                                                       GError** error)
{
	RygelExternalFreeDesktopPropertiesIface* _iface_;
	_iface_ = RYGEL_EXTERNAL_FREE_DESKTOP_PROPERTIES_GET_INTERFACE (self);
	if (_iface_->get_all_finish) {
		return _iface_->get_all_finish (self, _res_, error);
	}
	return NULL;
}

static void
rygel_external_free_desktop_properties_default_init (RygelExternalFreeDesktopPropertiesIface * iface,
                                                     gpointer iface_data)
{
}

 G_GNUC_NO_INLINE static GType
rygel_external_free_desktop_properties_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelExternalFreeDesktopPropertiesIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_external_free_desktop_properties_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType rygel_external_free_desktop_properties_type_id;
	rygel_external_free_desktop_properties_type_id = g_type_register_static (G_TYPE_INTERFACE, "RygelExternalFreeDesktopProperties", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (rygel_external_free_desktop_properties_type_id, G_TYPE_OBJECT);
	g_type_set_qdata (rygel_external_free_desktop_properties_type_id, g_quark_from_static_string ("vala-dbus-proxy-type"), (void*) rygel_external_free_desktop_properties_proxy_get_type);
	g_type_set_qdata (rygel_external_free_desktop_properties_type_id, g_quark_from_static_string ("vala-dbus-interface-name"), "org.freedesktop.DBus.Properties");
	g_type_set_qdata (rygel_external_free_desktop_properties_type_id, g_quark_from_static_string ("vala-dbus-interface-info"), (void*) (&_rygel_external_free_desktop_properties_dbus_interface_info));
	g_type_set_qdata (rygel_external_free_desktop_properties_type_id, g_quark_from_static_string ("vala-dbus-register-object"), (void*) rygel_external_free_desktop_properties_register_object);
	return rygel_external_free_desktop_properties_type_id;
}

GType
rygel_external_free_desktop_properties_get_type (void)
{
	static volatile gsize rygel_external_free_desktop_properties_type_id__once = 0;
	if (g_once_init_enter (&rygel_external_free_desktop_properties_type_id__once)) {
		GType rygel_external_free_desktop_properties_type_id;
		rygel_external_free_desktop_properties_type_id = rygel_external_free_desktop_properties_get_type_once ();
		g_once_init_leave (&rygel_external_free_desktop_properties_type_id__once, rygel_external_free_desktop_properties_type_id);
	}
	return rygel_external_free_desktop_properties_type_id__once;
}

G_DEFINE_TYPE_EXTENDED (RygelExternalFreeDesktopPropertiesProxy, rygel_external_free_desktop_properties_proxy, G_TYPE_DBUS_PROXY, 0, G_IMPLEMENT_INTERFACE (RYGEL_EXTERNAL_FREE_DESKTOP_TYPE_PROPERTIES, rygel_external_free_desktop_properties_proxy_rygel_external_free_desktop_properties_interface_init) )
static void
rygel_external_free_desktop_properties_proxy_class_init (RygelExternalFreeDesktopPropertiesProxyClass* klass)
{
	G_DBUS_PROXY_CLASS (klass)->g_signal = rygel_external_free_desktop_properties_proxy_g_signal;
}

static void
rygel_external_free_desktop_properties_proxy_g_signal (GDBusProxy* proxy,
                                                       const gchar* sender_name,
                                                       const gchar* signal_name,
                                                       GVariant* parameters)
{
}

static void
rygel_external_free_desktop_properties_proxy_init (RygelExternalFreeDesktopPropertiesProxy* self)
{
	g_dbus_proxy_set_interface_info (G_DBUS_PROXY (self), (GDBusInterfaceInfo *) (&_rygel_external_free_desktop_properties_dbus_interface_info));
}

static void
rygel_external_free_desktop_properties_proxy_get_all_async (RygelExternalFreeDesktopProperties* self,
                                                            const gchar* iface,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	G_DBUS_ERROR;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.DBus.Properties", "GetAll");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (iface));
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static GHashTable*
rygel_external_free_desktop_properties_proxy_get_all_finish (RygelExternalFreeDesktopProperties* self,
                                                             GAsyncResult* _res_,
                                                             GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	GVariant *_reply;
	GVariantIter _reply_iter;
	GHashTable* _result = NULL;
	GVariant* _tmp132_;
	GHashTable* _tmp133_;
	GVariantIter _tmp134_;
	GVariant* _tmp135_;
	GVariant* _tmp136_;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		return NULL;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		g_object_unref (_reply_message);
		return NULL;
	}
	_reply = g_dbus_message_get_body (_reply_message);
	g_variant_iter_init (&_reply_iter, _reply);
	_tmp132_ = g_variant_iter_next_value (&_reply_iter);
	_tmp133_ = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_variant_unref);
	g_variant_iter_init (&_tmp134_, _tmp132_);
	while (g_variant_iter_loop (&_tmp134_, "{?*}", &_tmp135_, &_tmp136_)) {
		g_hash_table_insert (_tmp133_, g_variant_dup_string (_tmp135_, NULL), g_variant_get_variant (_tmp136_));
	}
	_result = _tmp133_;
	g_variant_unref (_tmp132_);
	g_object_unref (_reply_message);
	return _result;
}

static void
rygel_external_free_desktop_properties_proxy_rygel_external_free_desktop_properties_interface_init (RygelExternalFreeDesktopPropertiesIface* iface)
{
	iface->get_all = rygel_external_free_desktop_properties_proxy_get_all_async;
	iface->get_all_finish = rygel_external_free_desktop_properties_proxy_get_all_finish;
}

static void
rygel_external_free_desktop_properties_dbus_interface_method_call (GDBusConnection* connection,
                                                                   const gchar* sender,
                                                                   const gchar* object_path,
                                                                   const gchar* interface_name,
                                                                   const gchar* method_name,
                                                                   GVariant* parameters,
                                                                   GDBusMethodInvocation* invocation,
                                                                   gpointer user_data)
{
	g_object_unref (invocation);
}

static GVariant*
rygel_external_free_desktop_properties_dbus_interface_get_property (GDBusConnection* connection,
                                                                    const gchar* sender,
                                                                    const gchar* object_path,
                                                                    const gchar* interface_name,
                                                                    const gchar* property_name,
                                                                    GError** error,
                                                                    gpointer user_data)
{
	return NULL;
}

static gboolean
rygel_external_free_desktop_properties_dbus_interface_set_property (GDBusConnection* connection,
                                                                    const gchar* sender,
                                                                    const gchar* object_path,
                                                                    const gchar* interface_name,
                                                                    const gchar* property_name,
                                                                    GVariant* value,
                                                                    GError** error,
                                                                    gpointer user_data)
{
	return FALSE;
}

guint
rygel_external_free_desktop_properties_register_object (gpointer object,
                                                        GDBusConnection* connection,
                                                        const gchar* path,
                                                        GError** error)
{
	guint result;
	gpointer *data;
	data = g_new (gpointer, 3);
	data[0] = g_object_ref (object);
	data[1] = g_object_ref (connection);
	data[2] = g_strdup (path);
	result = g_dbus_connection_register_object (connection, path, (GDBusInterfaceInfo *) (&_rygel_external_free_desktop_properties_dbus_interface_info), &_rygel_external_free_desktop_properties_dbus_interface_vtable, data, _rygel_external_free_desktop_properties_unregister_object, error);
	if (!result) {
		return 0;
	}
	return result;
}

static void
_rygel_external_free_desktop_properties_unregister_object (gpointer user_data)
{
	gpointer* data;
	data = user_data;
	g_object_unref (data[0]);
	g_object_unref (data[1]);
	g_free (data[2]);
	g_free (data);
}

