/* rygel-free-desktop-interfaces.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-free-desktop-interfaces.vala, do not modify */

/*
 * Copyright (C) 2009,2010 Nokia Corporation.
 *
 * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
 *                               <zeeshan.ali@nokia.com>
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include "rygel-server.h"

#define RYGEL_FREE_DESKTOP_DBUS_SERVICE "org.freedesktop.DBus"
#define RYGEL_FREE_DESKTOP_DBUS_OBJECT_PATH "/org/freedesktop/DBus"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define RYGEL_FREE_DESKTOP_TYPE_DBUS_OBJECT (rygel_free_desktop_dbus_object_get_type ())
#define RYGEL_FREE_DESKTOP_DBUS_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_FREE_DESKTOP_TYPE_DBUS_OBJECT, RygelFreeDesktopDBusObject))
#define RYGEL_FREE_DESKTOP_IS_DBUS_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_FREE_DESKTOP_TYPE_DBUS_OBJECT))
#define RYGEL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), RYGEL_FREE_DESKTOP_TYPE_DBUS_OBJECT, RygelFreeDesktopDBusObjectIface))

typedef struct _RygelFreeDesktopDBusObject RygelFreeDesktopDBusObject;
typedef struct _RygelFreeDesktopDBusObjectIface RygelFreeDesktopDBusObjectIface;

#define RYGEL_FREE_DESKTOP_TYPE_DBUS_OBJECT_PROXY (rygel_free_desktop_dbus_object_proxy_get_type ())
typedef GDBusProxy RygelFreeDesktopDBusObjectProxy;
typedef GDBusProxyClass RygelFreeDesktopDBusObjectProxyClass;

struct _RygelFreeDesktopDBusObjectIface {
	GTypeInterface parent_iface;
	void (*list_activatable_names) (RygelFreeDesktopDBusObject* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar** (*list_activatable_names_finish) (RygelFreeDesktopDBusObject* self, GAsyncResult* _res_, gint* result_length1, GError** error);
};

VALA_EXTERN GType rygel_free_desktop_dbus_object_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint rygel_free_desktop_dbus_object_register_object (void* object,
                                                      GDBusConnection* connection,
                                                      const gchar* path,
                                                      GError** error);
VALA_EXTERN GType rygel_free_desktop_dbus_object_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void rygel_free_desktop_dbus_object_list_activatable_names (RygelFreeDesktopDBusObject* self,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_);
VALA_EXTERN gchar** rygel_free_desktop_dbus_object_list_activatable_names_finish (RygelFreeDesktopDBusObject* self,
                                                                      GAsyncResult* _res_,
                                                                      gint* result_length1,
                                                                      GError** error);
static GType rygel_free_desktop_dbus_object_get_type_once (void);
static void rygel_free_desktop_dbus_object_proxy_g_signal (GDBusProxy* proxy,
                                                    const gchar* sender_name,
                                                    const gchar* signal_name,
                                                    GVariant* parameters);
static void _vala_g_async_ready_callback (GObject *source_object,
                                   GAsyncResult *res,
                                   void *user_data);
static void rygel_free_desktop_dbus_object_proxy_list_activatable_names_async (RygelFreeDesktopDBusObject* self,
                                                                        GAsyncReadyCallback _callback_,
                                                                        gpointer _user_data_);
static gchar** rygel_free_desktop_dbus_object_proxy_list_activatable_names_finish (RygelFreeDesktopDBusObject* self,
                                                                            GAsyncResult* _res_,
                                                                            gint* result_length1,
                                                                            GError** error);
static void rygel_free_desktop_dbus_object_proxy_rygel_free_desktop_dbus_object_interface_init (RygelFreeDesktopDBusObjectIface* iface);
static void rygel_free_desktop_dbus_object_dbus_interface_method_call (GDBusConnection* connection,
                                                                const gchar* sender,
                                                                const gchar* object_path,
                                                                const gchar* interface_name,
                                                                const gchar* method_name,
                                                                GVariant* parameters,
                                                                GDBusMethodInvocation* invocation,
                                                                gpointer user_data);
static GVariant* rygel_free_desktop_dbus_object_dbus_interface_get_property (GDBusConnection* connection,
                                                                      const gchar* sender,
                                                                      const gchar* object_path,
                                                                      const gchar* interface_name,
                                                                      const gchar* property_name,
                                                                      GError** error,
                                                                      gpointer user_data);
static gboolean rygel_free_desktop_dbus_object_dbus_interface_set_property (GDBusConnection* connection,
                                                                     const gchar* sender,
                                                                     const gchar* object_path,
                                                                     const gchar* interface_name,
                                                                     const gchar* property_name,
                                                                     GVariant* value,
                                                                     GError** error,
                                                                     gpointer user_data);
static void _rygel_free_desktop_dbus_object_unregister_object (gpointer user_data);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static const GDBusMethodInfo * const _rygel_free_desktop_dbus_object_dbus_method_info[] = {NULL};
static const GDBusSignalInfo * const _rygel_free_desktop_dbus_object_dbus_signal_info[] = {NULL};
static const GDBusPropertyInfo * const _rygel_free_desktop_dbus_object_dbus_property_info[] = {NULL};
static const GDBusInterfaceInfo _rygel_free_desktop_dbus_object_dbus_interface_info = {-1, "org.freedesktop.DBus", (GDBusMethodInfo **) (&_rygel_free_desktop_dbus_object_dbus_method_info), (GDBusSignalInfo **) (&_rygel_free_desktop_dbus_object_dbus_signal_info), (GDBusPropertyInfo **) (&_rygel_free_desktop_dbus_object_dbus_property_info), NULL};
static const GDBusInterfaceVTable _rygel_free_desktop_dbus_object_dbus_interface_vtable = {rygel_free_desktop_dbus_object_dbus_interface_method_call, rygel_free_desktop_dbus_object_dbus_interface_get_property, rygel_free_desktop_dbus_object_dbus_interface_set_property};

void
rygel_free_desktop_dbus_object_list_activatable_names (RygelFreeDesktopDBusObject* self,
                                                       GAsyncReadyCallback _callback_,
                                                       gpointer _user_data_)
{
	RygelFreeDesktopDBusObjectIface* _iface_;
	_iface_ = RYGEL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE (self);
	if (_iface_->list_activatable_names) {
		_iface_->list_activatable_names (self, _callback_, _user_data_);
	}
}

gchar**
rygel_free_desktop_dbus_object_list_activatable_names_finish (RygelFreeDesktopDBusObject* self,
                                                              GAsyncResult* _res_,
                                                              gint* result_length1,
                                                              GError** error)
{
	RygelFreeDesktopDBusObjectIface* _iface_;
	_iface_ = RYGEL_FREE_DESKTOP_DBUS_OBJECT_GET_INTERFACE (self);
	if (_iface_->list_activatable_names_finish) {
		return _iface_->list_activatable_names_finish (self, _res_, result_length1, error);
	}
	return NULL;
}

static void
rygel_free_desktop_dbus_object_default_init (RygelFreeDesktopDBusObjectIface * iface,
                                             gpointer iface_data)
{
}

 G_GNUC_NO_INLINE static GType
rygel_free_desktop_dbus_object_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelFreeDesktopDBusObjectIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_free_desktop_dbus_object_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType rygel_free_desktop_dbus_object_type_id;
	rygel_free_desktop_dbus_object_type_id = g_type_register_static (G_TYPE_INTERFACE, "RygelFreeDesktopDBusObject", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (rygel_free_desktop_dbus_object_type_id, G_TYPE_OBJECT);
	g_type_set_qdata (rygel_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-proxy-type"), (void*) rygel_free_desktop_dbus_object_proxy_get_type);
	g_type_set_qdata (rygel_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-interface-name"), "org.freedesktop.DBus");
	g_type_set_qdata (rygel_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-interface-info"), (void*) (&_rygel_free_desktop_dbus_object_dbus_interface_info));
	g_type_set_qdata (rygel_free_desktop_dbus_object_type_id, g_quark_from_static_string ("vala-dbus-register-object"), (void*) rygel_free_desktop_dbus_object_register_object);
	return rygel_free_desktop_dbus_object_type_id;
}

GType
rygel_free_desktop_dbus_object_get_type (void)
{
	static volatile gsize rygel_free_desktop_dbus_object_type_id__once = 0;
	if (g_once_init_enter (&rygel_free_desktop_dbus_object_type_id__once)) {
		GType rygel_free_desktop_dbus_object_type_id;
		rygel_free_desktop_dbus_object_type_id = rygel_free_desktop_dbus_object_get_type_once ();
		g_once_init_leave (&rygel_free_desktop_dbus_object_type_id__once, rygel_free_desktop_dbus_object_type_id);
	}
	return rygel_free_desktop_dbus_object_type_id__once;
}

G_DEFINE_TYPE_EXTENDED (RygelFreeDesktopDBusObjectProxy, rygel_free_desktop_dbus_object_proxy, G_TYPE_DBUS_PROXY, 0, G_IMPLEMENT_INTERFACE (RYGEL_FREE_DESKTOP_TYPE_DBUS_OBJECT, rygel_free_desktop_dbus_object_proxy_rygel_free_desktop_dbus_object_interface_init) )
static void
rygel_free_desktop_dbus_object_proxy_class_init (RygelFreeDesktopDBusObjectProxyClass* klass)
{
	G_DBUS_PROXY_CLASS (klass)->g_signal = rygel_free_desktop_dbus_object_proxy_g_signal;
}

static void
rygel_free_desktop_dbus_object_proxy_g_signal (GDBusProxy* proxy,
                                               const gchar* sender_name,
                                               const gchar* signal_name,
                                               GVariant* parameters)
{
}

static void
rygel_free_desktop_dbus_object_proxy_init (RygelFreeDesktopDBusObjectProxy* self)
{
	g_dbus_proxy_set_interface_info (G_DBUS_PROXY (self), (GDBusInterfaceInfo *) (&_rygel_free_desktop_dbus_object_dbus_interface_info));
}

static void
_vala_g_async_ready_callback (GObject *source_object,
                              GAsyncResult *res,
                              void *user_data)
{
	g_task_return_pointer (user_data, g_object_ref (res), g_object_unref);
	g_object_unref (user_data);
}

static void
rygel_free_desktop_dbus_object_proxy_list_activatable_names_async (RygelFreeDesktopDBusObject* self,
                                                                   GAsyncReadyCallback _callback_,
                                                                   gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	G_DBUS_ERROR;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.DBus", "ListActivatableNames");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static gchar**
rygel_free_desktop_dbus_object_proxy_list_activatable_names_finish (RygelFreeDesktopDBusObject* self,
                                                                    GAsyncResult* _res_,
                                                                    gint* result_length1,
                                                                    GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	GVariant *_reply;
	GVariantIter _reply_iter;
	gchar** _result = NULL;
	gint _result_length1;
	GVariant* _tmp20_;
	gchar** _tmp21_;
	gint _tmp21__length;
	gint _tmp21__size;
	gint _tmp21__length1;
	GVariantIter _tmp22_;
	GVariant* _tmp23_;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		return NULL;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		g_object_unref (_reply_message);
		return NULL;
	}
	_reply = g_dbus_message_get_body (_reply_message);
	g_variant_iter_init (&_reply_iter, _reply);
	_result_length1 = 0;
	_tmp20_ = g_variant_iter_next_value (&_reply_iter);
	_tmp21_ = g_new (gchar*, 5);
	_tmp21__length = 0;
	_tmp21__size = 4;
	_tmp21__length1 = 0;
	g_variant_iter_init (&_tmp22_, _tmp20_);
	for (; (_tmp23_ = g_variant_iter_next_value (&_tmp22_)) != NULL; _tmp21__length1++) {
		if (_tmp21__size == _tmp21__length) {
			_tmp21__size = 2 * _tmp21__size;
			_tmp21_ = g_renew (gchar*, _tmp21_, _tmp21__size + 1);
		}
		_tmp21_[_tmp21__length++] = g_variant_dup_string (_tmp23_, NULL);
		g_variant_unref (_tmp23_);
	}
	_result_length1 = _tmp21__length1;
	_tmp21_[_tmp21__length] = NULL;
	_result = _tmp21_;
	g_variant_unref (_tmp20_);
	*result_length1 = _result_length1;
	g_object_unref (_reply_message);
	return _result;
}

static void
rygel_free_desktop_dbus_object_proxy_rygel_free_desktop_dbus_object_interface_init (RygelFreeDesktopDBusObjectIface* iface)
{
	iface->list_activatable_names = rygel_free_desktop_dbus_object_proxy_list_activatable_names_async;
	iface->list_activatable_names_finish = rygel_free_desktop_dbus_object_proxy_list_activatable_names_finish;
}

static void
rygel_free_desktop_dbus_object_dbus_interface_method_call (GDBusConnection* connection,
                                                           const gchar* sender,
                                                           const gchar* object_path,
                                                           const gchar* interface_name,
                                                           const gchar* method_name,
                                                           GVariant* parameters,
                                                           GDBusMethodInvocation* invocation,
                                                           gpointer user_data)
{
	g_object_unref (invocation);
}

static GVariant*
rygel_free_desktop_dbus_object_dbus_interface_get_property (GDBusConnection* connection,
                                                            const gchar* sender,
                                                            const gchar* object_path,
                                                            const gchar* interface_name,
                                                            const gchar* property_name,
                                                            GError** error,
                                                            gpointer user_data)
{
	return NULL;
}

static gboolean
rygel_free_desktop_dbus_object_dbus_interface_set_property (GDBusConnection* connection,
                                                            const gchar* sender,
                                                            const gchar* object_path,
                                                            const gchar* interface_name,
                                                            const gchar* property_name,
                                                            GVariant* value,
                                                            GError** error,
                                                            gpointer user_data)
{
	return FALSE;
}

guint
rygel_free_desktop_dbus_object_register_object (gpointer object,
                                                GDBusConnection* connection,
                                                const gchar* path,
                                                GError** error)
{
	guint result;
	gpointer *data;
	data = g_new (gpointer, 3);
	data[0] = g_object_ref (object);
	data[1] = g_object_ref (connection);
	data[2] = g_strdup (path);
	result = g_dbus_connection_register_object (connection, path, (GDBusInterfaceInfo *) (&_rygel_free_desktop_dbus_object_dbus_interface_info), &_rygel_free_desktop_dbus_object_dbus_interface_vtable, data, _rygel_free_desktop_dbus_object_unregister_object, error);
	if (!result) {
		return 0;
	}
	return result;
}

static void
_rygel_free_desktop_dbus_object_unregister_object (gpointer user_data)
{
	gpointer* data;
	data = user_data;
	g_object_unref (data[0]);
	g_object_unref (data[1]);
	g_free (data[2]);
	g_free (data);
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

