/* raumfeld-hacks.c generated by valac 0.56.19, the Vala compiler
 * generated from raumfeld-hacks.vala, do not modify */

/* SPDX-License-Identifier: LGPL-2.1-or-later*/
/* SPDX-FileCopyrightText: 2003 Jens Georg <mail@jensge.org>*/

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "rygel-server.h"
#include <gio/gio.h>
#include <libsoup/soup.h>
#include <gee.h>

#define RYGEL_RAUMFELD_HACKS_AGENT ".*Raum[fF]eld.*|.*souphttpsrc.*"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define RYGEL_TYPE_CLIENT_HACKS (rygel_client_hacks_get_type ())
#define RYGEL_CLIENT_HACKS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_CLIENT_HACKS, RygelClientHacks))
#define RYGEL_CLIENT_HACKS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_CLIENT_HACKS, RygelClientHacksClass))
#define RYGEL_IS_CLIENT_HACKS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_CLIENT_HACKS))
#define RYGEL_IS_CLIENT_HACKS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_CLIENT_HACKS))
#define RYGEL_CLIENT_HACKS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_CLIENT_HACKS, RygelClientHacksClass))

typedef struct _RygelClientHacks RygelClientHacks;
typedef struct _RygelClientHacksClass RygelClientHacksClass;
typedef struct _RygelClientHacksPrivate RygelClientHacksPrivate;

#define RYGEL_TYPE_MEDIA_QUERY_ACTION (rygel_media_query_action_get_type ())
#define RYGEL_MEDIA_QUERY_ACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_MEDIA_QUERY_ACTION, RygelMediaQueryAction))
#define RYGEL_MEDIA_QUERY_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_MEDIA_QUERY_ACTION, RygelMediaQueryActionClass))
#define RYGEL_IS_MEDIA_QUERY_ACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_MEDIA_QUERY_ACTION))
#define RYGEL_IS_MEDIA_QUERY_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_MEDIA_QUERY_ACTION))
#define RYGEL_MEDIA_QUERY_ACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_MEDIA_QUERY_ACTION, RygelMediaQueryActionClass))

typedef struct _RygelMediaQueryAction RygelMediaQueryAction;
typedef struct _RygelMediaQueryActionClass RygelMediaQueryActionClass;

#define RYGEL_TYPE_RAUMFELD_HACKS (rygel_raumfeld_hacks_get_type ())
#define RYGEL_RAUMFELD_HACKS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_RAUMFELD_HACKS, RygelRaumfeldHacks))
#define RYGEL_RAUMFELD_HACKS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_TYPE_RAUMFELD_HACKS, RygelRaumfeldHacksClass))
#define RYGEL_IS_RAUMFELD_HACKS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_TYPE_RAUMFELD_HACKS))
#define RYGEL_IS_RAUMFELD_HACKS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_TYPE_RAUMFELD_HACKS))
#define RYGEL_RAUMFELD_HACKS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_TYPE_RAUMFELD_HACKS, RygelRaumfeldHacksClass))

typedef struct _RygelRaumfeldHacks RygelRaumfeldHacks;
typedef struct _RygelRaumfeldHacksClass RygelRaumfeldHacksClass;
typedef struct _RygelRaumfeldHacksPrivate RygelRaumfeldHacksPrivate;
enum  {
	RYGEL_RAUMFELD_HACKS_0_PROPERTY,
	RYGEL_RAUMFELD_HACKS_NUM_PROPERTIES
};
static GParamSpec* rygel_raumfeld_hacks_properties[RYGEL_RAUMFELD_HACKS_NUM_PROPERTIES];

#define RYGEL_TYPE_CLIENT_HACKS_ERROR (rygel_client_hacks_error_get_type ())
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _RygelClientHacks {
	GObject parent_instance;
	RygelClientHacksPrivate * priv;
	GRegex* agent_regex;
};

struct _RygelClientHacksClass {
	GObjectClass parent_class;
	void (*translate_container_id) (RygelClientHacks* self, RygelMediaQueryAction* action, gchar** container_id);
	void (*apply) (RygelClientHacks* self, RygelMediaObject* object);
	void (*filter_sort_criteria) (RygelClientHacks* self, gchar** sort_criteria);
	gboolean (*force_seek) (RygelClientHacks* self);
	void (*modify_headers) (RygelClientHacks* self, RygelHTTPRequest* request);
	void (*search) (RygelClientHacks* self, RygelSearchableContainer* container, RygelSearchExpression* expression, guint offset, guint max_count, const gchar* sort_criteria, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	RygelMediaObjects* (*search_finish) (RygelClientHacks* self, GAsyncResult* _res_, guint* total_matches, GError** error);
};

struct _RygelRaumfeldHacks {
	RygelClientHacks parent_instance;
	RygelRaumfeldHacksPrivate * priv;
};

struct _RygelRaumfeldHacksClass {
	RygelClientHacksClass parent_class;
};

typedef enum  {
	RYGEL_CLIENT_HACKS_ERROR_NA
} RygelClientHacksError;
#define RYGEL_CLIENT_HACKS_ERROR rygel_client_hacks_error_quark ()

static gpointer rygel_raumfeld_hacks_parent_class = NULL;

VALA_EXTERN GType rygel_client_hacks_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType rygel_media_query_action_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType rygel_raumfeld_hacks_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void rygel_client_hacks_apply (RygelClientHacks* self,
                               RygelMediaObject* object);
VALA_EXTERN GQuark rygel_client_hacks_error_quark (void);
VALA_EXTERN GType rygel_client_hacks_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN RygelRaumfeldHacks* rygel_raumfeld_hacks_new (SoupServerMessage* message,
                                              GError** error);
VALA_EXTERN RygelRaumfeldHacks* rygel_raumfeld_hacks_construct (GType object_type,
                                                    SoupServerMessage* message,
                                                    GError** error);
VALA_EXTERN RygelClientHacks* rygel_client_hacks_construct (GType object_type,
                                                const gchar* agent,
                                                SoupServerMessage* message,
                                                GError** error);
static void rygel_raumfeld_hacks_real_apply (RygelClientHacks* base,
                                      RygelMediaObject* object);
static GType rygel_raumfeld_hacks_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

RygelRaumfeldHacks*
rygel_raumfeld_hacks_construct (GType object_type,
                                SoupServerMessage* message,
                                GError** error)
{
	RygelRaumfeldHacks * self = NULL;
	GError* _inner_error0_ = NULL;
	self = (RygelRaumfeldHacks*) rygel_client_hacks_construct (object_type, RYGEL_RAUMFELD_HACKS_AGENT, message, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == RYGEL_CLIENT_HACKS_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_object_unref0 (self);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	return self;
}

RygelRaumfeldHacks*
rygel_raumfeld_hacks_new (SoupServerMessage* message,
                          GError** error)
{
	return rygel_raumfeld_hacks_construct (RYGEL_TYPE_RAUMFELD_HACKS, message, error);
}

static void
rygel_raumfeld_hacks_real_apply (RygelClientHacks* base,
                                 RygelMediaObject* object)
{
	RygelRaumfeldHacks * self;
	self = (RygelRaumfeldHacks*) base;
	g_return_if_fail (object != NULL);
	{
		GeeList* _resource_list = NULL;
		GeeList* _tmp0_;
		gint _resource_size = 0;
		GeeList* _tmp1_;
		gint _tmp2_;
		gint _tmp3_;
		gint _resource_index = 0;
		_tmp0_ = rygel_media_object_get_resource_list (object);
		_resource_list = _tmp0_;
		_tmp1_ = _resource_list;
		_tmp2_ = gee_collection_get_size ((GeeCollection*) _tmp1_);
		_tmp3_ = _tmp2_;
		_resource_size = _tmp3_;
		_resource_index = -1;
		while (TRUE) {
			gint _tmp4_;
			gint _tmp5_;
			RygelMediaResource* resource = NULL;
			GeeList* _tmp6_;
			gpointer _tmp7_;
			gboolean _tmp8_ = FALSE;
			RygelMediaResource* _tmp9_;
			const gchar* _tmp10_;
			const gchar* _tmp11_;
			_resource_index = _resource_index + 1;
			_tmp4_ = _resource_index;
			_tmp5_ = _resource_size;
			if (!(_tmp4_ < _tmp5_)) {
				break;
			}
			_tmp6_ = _resource_list;
			_tmp7_ = gee_list_get (_tmp6_, _resource_index);
			resource = (RygelMediaResource*) _tmp7_;
			_tmp9_ = resource;
			_tmp10_ = rygel_media_resource_get_mime_type (_tmp9_);
			_tmp11_ = _tmp10_;
			if (g_strcmp0 (_tmp11_, "audio/x-vorbis+ogg") == 0) {
				_tmp8_ = TRUE;
			} else {
				RygelMediaResource* _tmp12_;
				const gchar* _tmp13_;
				const gchar* _tmp14_;
				_tmp12_ = resource;
				_tmp13_ = rygel_media_resource_get_mime_type (_tmp12_);
				_tmp14_ = _tmp13_;
				_tmp8_ = g_strcmp0 (_tmp14_, "audio/x-flac+ogg") == 0;
			}
			if (_tmp8_) {
				RygelMediaResource* _tmp15_;
				_tmp15_ = resource;
				rygel_media_resource_set_mime_type (_tmp15_, "audio/ogg");
			}
			_g_object_unref0 (resource);
		}
		_g_object_unref0 (_resource_list);
	}
}

static void
rygel_raumfeld_hacks_class_init (RygelRaumfeldHacksClass * klass,
                                 gpointer klass_data)
{
	rygel_raumfeld_hacks_parent_class = g_type_class_peek_parent (klass);
	((RygelClientHacksClass *) klass)->apply = (void (*) (RygelClientHacks*, RygelMediaObject*)) rygel_raumfeld_hacks_real_apply;
}

static void
rygel_raumfeld_hacks_instance_init (RygelRaumfeldHacks * self,
                                    gpointer klass)
{
}

 G_GNUC_NO_INLINE static GType
rygel_raumfeld_hacks_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (RygelRaumfeldHacksClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_raumfeld_hacks_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RygelRaumfeldHacks), 0, (GInstanceInitFunc) rygel_raumfeld_hacks_instance_init, NULL };
	GType rygel_raumfeld_hacks_type_id;
	rygel_raumfeld_hacks_type_id = g_type_register_static (RYGEL_TYPE_CLIENT_HACKS, "RygelRaumfeldHacks", &g_define_type_info, 0);
	return rygel_raumfeld_hacks_type_id;
}

GType
rygel_raumfeld_hacks_get_type (void)
{
	static volatile gsize rygel_raumfeld_hacks_type_id__once = 0;
	if (g_once_init_enter (&rygel_raumfeld_hacks_type_id__once)) {
		GType rygel_raumfeld_hacks_type_id;
		rygel_raumfeld_hacks_type_id = rygel_raumfeld_hacks_get_type_once ();
		g_once_init_leave (&rygel_raumfeld_hacks_type_id__once, rygel_raumfeld_hacks_type_id);
	}
	return rygel_raumfeld_hacks_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

