/* rygel-example-renderer-plugin-vala.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __EXAMPLES_RENDERER_PLUGINS_VALA_RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA_H__
#define __EXAMPLES_RENDERER_PLUGINS_VALA_RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA_H__

#include "rygel-core.h"
#include "rygel-renderer.h"
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

G_BEGIN_DECLS

#define RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA_NAME "ExampleRendererPluginVala"
#define RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA_TITLE "Example Renderer Plugin Vala"
#define RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA_DESCRIPTION "An example Rygel renderer plugin implemented in vala."
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define RYGEL_EXAMPLE_TYPE_RENDERER_PLUGIN_VALA (rygel_example_renderer_plugin_vala_get_type ())
#define RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_EXAMPLE_TYPE_RENDERER_PLUGIN_VALA, RygelExampleRendererPluginVala))
#define RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_EXAMPLE_TYPE_RENDERER_PLUGIN_VALA, RygelExampleRendererPluginValaClass))
#define RYGEL_EXAMPLE_IS_RENDERER_PLUGIN_VALA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_EXAMPLE_TYPE_RENDERER_PLUGIN_VALA))
#define RYGEL_EXAMPLE_IS_RENDERER_PLUGIN_VALA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_EXAMPLE_TYPE_RENDERER_PLUGIN_VALA))
#define RYGEL_EXAMPLE_RENDERER_PLUGIN_VALA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_EXAMPLE_TYPE_RENDERER_PLUGIN_VALA, RygelExampleRendererPluginValaClass))

typedef struct _RygelExampleRendererPluginVala RygelExampleRendererPluginVala;
typedef struct _RygelExampleRendererPluginValaClass RygelExampleRendererPluginValaClass;
typedef struct _RygelExampleRendererPluginValaPrivate RygelExampleRendererPluginValaPrivate;

#define RYGEL_EXAMPLE_TYPE_PLAYER_VALA (rygel_example_player_vala_get_type ())
#define RYGEL_EXAMPLE_PLAYER_VALA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_EXAMPLE_TYPE_PLAYER_VALA, RygelExamplePlayerVala))
#define RYGEL_EXAMPLE_PLAYER_VALA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_EXAMPLE_TYPE_PLAYER_VALA, RygelExamplePlayerValaClass))
#define RYGEL_EXAMPLE_IS_PLAYER_VALA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_EXAMPLE_TYPE_PLAYER_VALA))
#define RYGEL_EXAMPLE_IS_PLAYER_VALA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_EXAMPLE_TYPE_PLAYER_VALA))
#define RYGEL_EXAMPLE_PLAYER_VALA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_EXAMPLE_TYPE_PLAYER_VALA, RygelExamplePlayerValaClass))

typedef struct _RygelExamplePlayerVala RygelExamplePlayerVala;
typedef struct _RygelExamplePlayerValaClass RygelExamplePlayerValaClass;
typedef struct _RygelExamplePlayerValaPrivate RygelExamplePlayerValaPrivate;

struct _RygelExampleRendererPluginVala {
	RygelMediaRendererPlugin parent_instance;
	RygelExampleRendererPluginValaPrivate * priv;
};

struct _RygelExampleRendererPluginValaClass {
	RygelMediaRendererPluginClass parent_class;
};

struct _RygelExamplePlayerVala {
	GObject parent_instance;
	RygelExamplePlayerValaPrivate * priv;
};

struct _RygelExamplePlayerValaClass {
	GObjectClass parent_class;
};

VALA_EXTERN void module_init (RygelPluginLoader* loader);
VALA_EXTERN GType rygel_example_renderer_plugin_vala_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelExampleRendererPluginVala, g_object_unref)
VALA_EXTERN RygelExampleRendererPluginVala* rygel_example_renderer_plugin_vala_new (void);
VALA_EXTERN RygelExampleRendererPluginVala* rygel_example_renderer_plugin_vala_construct (GType object_type);
VALA_EXTERN GType rygel_example_player_vala_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (RygelExamplePlayerVala, g_object_unref)
VALA_EXTERN RygelExamplePlayerVala* rygel_example_player_vala_get_default (void);
VALA_EXTERN RygelExamplePlayerVala* rygel_example_player_vala_new (void);
VALA_EXTERN RygelExamplePlayerVala* rygel_example_player_vala_construct (GType object_type);

G_END_DECLS

#endif
