/* rygel-ruih-module.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-ruih-module.vala, do not modify */

/*
 * Copyright (C) 2013  Cable Television Laboratories, Inc.
 *
 * Author: Neha Shanbhag <N.Shanbhag@cablelabs.com>
 * Contact: http://www.cablelabs.com/
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "rygel-ruih-plugin.h"
#include "rygel-core.h"
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include "rygel-ruih.h"
#include <glib-object.h>

#define RYGEL_RUIH_PLUGIN_NAME "Ruih"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define RYGEL_RUIH_TYPE_PLUGIN (rygel_ruih_plugin_get_type ())
#define RYGEL_RUIH_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_RUIH_TYPE_PLUGIN, RygelRuihPlugin))
#define RYGEL_RUIH_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RYGEL_RUIH_TYPE_PLUGIN, RygelRuihPluginClass))
#define RYGEL_RUIH_IS_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RYGEL_RUIH_TYPE_PLUGIN))
#define RYGEL_RUIH_IS_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RYGEL_RUIH_TYPE_PLUGIN))
#define RYGEL_RUIH_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RYGEL_RUIH_TYPE_PLUGIN, RygelRuihPluginClass))

typedef struct _RygelRuihPlugin RygelRuihPlugin;
typedef struct _RygelRuihPluginClass RygelRuihPluginClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

VALA_EXTERN GType rygel_ruih_plugin_get_type (void) G_GNUC_CONST ;
VALA_EXTERN RygelRuihPlugin* rygel_ruih_plugin_new (void);
VALA_EXTERN RygelRuihPlugin* rygel_ruih_plugin_construct (GType object_type);

void
module_init (RygelPluginLoader* loader)
{
	RygelRuihPlugin* plugin = NULL;
	RygelRuihPlugin* _tmp0_;
	RygelRuihPlugin* _tmp1_;
	g_return_if_fail (loader != NULL);
	if (rygel_plugin_loader_plugin_disabled (loader, RYGEL_RUIH_PLUGIN_NAME)) {
		g_message ("rygel-ruih-module.vala:28: Plugin '%s' disabled by user, ignoring..", RYGEL_RUIH_PLUGIN_NAME);
		return;
	}
	_tmp0_ = rygel_ruih_plugin_new ();
	plugin = _tmp0_;
	_tmp1_ = plugin;
	rygel_plugin_loader_add_plugin (loader, (RygelPlugin*) _tmp1_);
	_g_object_unref0 (plugin);
}

