/* rygel-ruih-wildcard-filter-entry.c generated by valac 0.56.19, the Vala compiler
 * generated from rygel-ruih-wildcard-filter-entry.vala, do not modify */

/*
 * Copyright (C) 2013  Cable Television Laboratories, Inc.
 *
 * Author: Neha Shanbhag <N.Shanbhag@cablelabs.com>
 * Contact: http://www.cablelabs.com/
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "rygel-ruih.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static gpointer wild_card_filter_entry_parent_class = NULL;

static gboolean wild_card_filter_entry_real_matches (FilterEntry* base,
                                              const gchar* name,
                                              const gchar* value);
static GType wild_card_filter_entry_get_type_once (void);

WildCardFilterEntry*
wild_card_filter_entry_construct (GType object_type)
{
	WildCardFilterEntry* self = NULL;
	self = (WildCardFilterEntry*) filter_entry_construct (object_type, "*", "*");
	return self;
}

WildCardFilterEntry*
wild_card_filter_entry_new (void)
{
	return wild_card_filter_entry_construct (TYPE_WILD_CARD_FILTER_ENTRY);
}

static gboolean
wild_card_filter_entry_real_matches (FilterEntry* base,
                                     const gchar* name,
                                     const gchar* value)
{
	WildCardFilterEntry * self;
	gboolean result;
	self = (WildCardFilterEntry*) base;
	g_return_val_if_fail (name != NULL, FALSE);
	g_return_val_if_fail (value != NULL, FALSE);
	result = TRUE;
	return result;
}

static void
wild_card_filter_entry_class_init (WildCardFilterEntryClass * klass,
                                   gpointer klass_data)
{
	wild_card_filter_entry_parent_class = g_type_class_peek_parent (klass);
	((FilterEntryClass *) klass)->matches = (gboolean (*) (FilterEntry*, const gchar*, const gchar*)) wild_card_filter_entry_real_matches;
}

static void
wild_card_filter_entry_instance_init (WildCardFilterEntry * self,
                                      gpointer klass)
{
}

 G_GNUC_NO_INLINE static GType
wild_card_filter_entry_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (WildCardFilterEntryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) wild_card_filter_entry_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (WildCardFilterEntry), 0, (GInstanceInitFunc) wild_card_filter_entry_instance_init, NULL };
	GType wild_card_filter_entry_type_id;
	wild_card_filter_entry_type_id = g_type_register_static (TYPE_FILTER_ENTRY, "WildCardFilterEntry", &g_define_type_info, 0);
	return wild_card_filter_entry_type_id;
}

GType
wild_card_filter_entry_get_type (void)
{
	static volatile gsize wild_card_filter_entry_type_id__once = 0;
	if (g_once_init_enter (&wild_card_filter_entry_type_id__once)) {
		GType wild_card_filter_entry_type_id;
		wild_card_filter_entry_type_id = wild_card_filter_entry_get_type_once ();
		g_once_init_leave (&wild_card_filter_entry_type_id__once, wild_card_filter_entry_type_id);
	}
	return wild_card_filter_entry_type_id__once;
}

